# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import shutil
import tempfile

from unittest import TestCase
from mock import patch

from managesf.tests import dummy_conf

from managesf.model.yamlbkd import engine
from managesf.model.yamlbkd.engine import SFResourceBackendEngine
from managesf.model.yamlbkd.engine import ResourceDepsException
from managesf.model.yamlbkd.engine import ResourceUnicityException
from managesf.model.yamlbkd.engine import RTYPENotDefinedException

from managesf.model.yamlbkd.yamlbackend import YAMLDBException
from managesf.model.yamlbkd.resource import BaseResource
from managesf.model.yamlbkd.resource import ModelInvalidException
from managesf.model.yamlbkd.resource import ResourceInvalidException
from managesf.model.yamlbkd.resources.dummy import Dummy


class EngineTest(TestCase):

    @classmethod
    def setupClass(cls):
        cls.conf = dummy_conf()
        engine.conf = cls.conf

    def setUp(self):
        self.to_delete = []

    def tearDown(self):
        for d in self.to_delete:
            shutil.rmtree(d)

    def test_init_engine(self):
        SFResourceBackendEngine('/tmp/dir',
                                'resources')

    def test_get_resources_priority(self):
        class A(BaseResource):
            PRIORITY = 60
            PRIMARY_KEY = None

        class B(BaseResource):
            PRIORITY = 40
            PRIMARY_KEY = None

        class C(BaseResource):
            PRIORITY = 55
            PRIMARY_KEY = None

        en = SFResourceBackendEngine(None, None)
        with patch.dict(engine.MAPPING,
                        {'dummies': Dummy,
                         'A': A,
                         'B': B,
                         'C': C}, clear=True):
            # Resource callback will be called in that
            # order A, C, dummies, B
            self.assertEqual([('A', 60),
                              ('C', 55),
                              ('dummies', 50),
                              ('B', 40)],
                             en._get_resources_priority())
            self.assertTrue(len(en._get_resources_priority()), 4)

    def test_load_resource_data(self):
        path = tempfile.mkdtemp()
        self.to_delete.append(path)
        with patch('managesf.model.yamlbkd.yamlbackend.'
                   'YAMLBackend.__init__') as i, \
                patch('managesf.model.yamlbkd.yamlbackend.'
                      'YAMLBackend.get_data') as g:
            i.return_value = None
            g.return_value = {}
            en = SFResourceBackendEngine(path,
                                         'resources')
            en._load_resource_data(
                'http://sftests.com/r/config.git',
                'heads/master', 'mark')
        self.assertTrue(os.path.isdir(
            os.path.join(path, 'mark')))
        self.assertTrue(i.called)
        self.assertTrue(g.called)

    def test_load_resources_data(self):
        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resource_data') as lrd:
            lrd.return_value = {}
            en = SFResourceBackendEngine(None, None)
            en._load_resources_data(
                'http://sftests.com/r/config.git',
                'heads/master',
                'http://sftests.com/r/config.git',
                'changes/99/899/1')
        self.assertEqual(len(lrd.mock_calls), 2)

    def test_validate(self):
        path = tempfile.mkdtemp()
        self.to_delete.append(path)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resources_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as vc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:
            lrd.return_value = (None, None)
            eng = SFResourceBackendEngine(path, None)
            status, _ = eng.validate(None, None, None, None)
            self.assertTrue(lrd.called)
            self.assertTrue(gdd.called)
            self.assertTrue(cdc.called)
            self.assertTrue(cu.called)
            self.assertTrue(vc.called)
            self.assertTrue(cd.called)
            self.assertTrue(status)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resources_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as vc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:
            lrd.side_effect = YAMLDBException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.validate(None, None, None, None)
            self.assertEqual(len(logs), 1)
            self.assertFalse(status)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resources_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as vc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:
            lrd.return_value = (None, None)
            vc.side_effect = ResourceInvalidException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.validate(None, None, None, None)
            self.assertEqual(len(logs), 1)
            self.assertFalse(status)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resources_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as vc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:

            lrd.return_value = (None, None)
            vc.side_effect = ResourceDepsException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.validate(None, None, None, None)
            self.assertEqual(len(logs), 1)
            self.assertFalse(status)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resources_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as vc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:

            lrd.return_value = (None, None)
            cu.side_effect = ResourceUnicityException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.validate(None, None, None, None)
            self.assertEqual(len(logs), 1)
            self.assertFalse(status)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resources_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as vc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:

            lrd.return_value = (None, None)
            cd.side_effect = RTYPENotDefinedException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.validate(None, None, None, None)
            self.assertEqual(len(logs), 1)
            self.assertFalse(status)

    def test_validate_from_structured_data(self):
        path = tempfile.mkdtemp()
        self.to_delete.append(path)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resource_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_load_resource_data_from_memory') as lm, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as vc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:
            lrd.return_value = (None, None)
            eng = SFResourceBackendEngine(path, None)
            status, _ = eng.validate_from_structured_data(
                None, None, None)
            self.assertTrue(lrd.called)
            self.assertTrue(cdc.called)
            self.assertTrue(cu.called)
            self.assertTrue(lm.called)
            self.assertTrue(gdd.called)
            self.assertTrue(vc.called)
            self.assertTrue(cd.called)
            self.assertTrue(status)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resource_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_load_resource_data_from_memory') as lm, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as vc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:
            lrd.side_effect = YAMLDBException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.validate_from_structured_data(
                None, None, None)
            self.assertEqual(len(logs), 1)
            self.assertFalse(status)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resource_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_load_resource_data_from_memory') as lm, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as v, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:
            lrd.return_value = (None, None)
            v.side_effect = ResourceInvalidException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.validate_from_structured_data(
                None, None, None)
            self.assertEqual(len(logs), 1)
            self.assertFalse(status)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resource_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_load_resource_data_from_memory') as lm, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as v, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:
            lrd.return_value = (None, None)
            v.side_effect = ResourceInvalidException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.validate_from_structured_data(
                None, None, None)
            self.assertEqual(len(logs), 1)
            self.assertFalse(status)

        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resource_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cdc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_load_resource_data_from_memory') as lm, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as v, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_rtype_defined') as cd:
            lrd.return_value = (None, None)
            cd.side_effect = RTYPENotDefinedException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.validate_from_structured_data(
                None, None, None)
            self.assertEqual(len(logs), 1)
            self.assertFalse(status)

    def test_apply(self):
        path = tempfile.mkdtemp()
        self.to_delete.append(path)
        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resources_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.'
                      'SFResourceBackendEngine._get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_resolv_resources_need_refresh') as rrnr, \
                patch('managesf.model.yamlbkd.engine.'
                      'SFResourceBackendEngine._apply_changes') as ac:
            lrd.return_value = (None, None)
            ac.return_value = False
            rrnr.return_value = []
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.apply(None, None, None, None)
            self.assertTrue(lrd.called)
            self.assertTrue(gdd.called)
            self.assertTrue(rrnr.called)
            self.assertTrue(ac.called)
            self.assertTrue(status)
        with patch('managesf.model.yamlbkd.engine.'
                   'SFResourceBackendEngine._load_resources_data') as lrd, \
                patch('managesf.model.yamlbkd.engine.'
                      'SFResourceBackendEngine._get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_resolv_resources_need_refresh') as rrnr, \
                patch('managesf.model.yamlbkd.engine.'
                      'SFResourceBackendEngine._apply_changes') as ac:
            lrd.side_effect = YAMLDBException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.apply(None, None, None, None)
            self.assertEqual(len(logs), 1)
            self.assertFalse(status)

    def test_direct_apply(self):
        path = tempfile.mkdtemp()
        self.to_delete.append(path)
        with patch('yaml.safe_load'), \
                patch('managesf.model.yamlbkd.yamlbackend.YAMLBackend.'
                      '_validate_base_struct'), \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as gdd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as vc, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_resolv_resources_need_refresh') as rrnr, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_apply_changes') as ac:
            ac.return_value = False
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.direct_apply(None, None)
            self.assertTrue(status)
            self.assertEqual(len(logs), 0)
            self.assertTrue(gdd.called)
            self.assertTrue(cd.called)
            self.assertTrue(cu.called)
            self.assertTrue(vc.called)
            self.assertTrue(rrnr.called)
            self.assertTrue(ac.called)
            self.assertTrue(status)

        with patch('yaml.safe_load'), \
                patch('managesf.model.yamlbkd.yamlbackend.YAMLBackend.'
                      '_validate_base_struct'), \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_get_data_diff') as g, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_deps_constraints') as cd, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_check_unicity_constraints') as cu, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_validate_changes') as v, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_resolv_resources_need_refresh') as r, \
                patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                      '_apply_changes') as a:
            v.side_effect = ResourceInvalidException('')
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.direct_apply(None, None)
            self.assertFalse(status)
            self.assertEqual(len(logs), 1)
            self.assertTrue(g.called)
            self.assertTrue(cd.called)
            self.assertTrue(cu.called)
            self.assertFalse(r.called)
            self.assertFalse(a.called)
        with patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                   '_apply_changes') as a, \
                patch.dict(engine.MAPPING, {'dummies': Dummy},
                           clear=True):
            a.return_value = False
            prev = "resources: {}"
            new = """resources:
  dummies:
    id1:
      name: dum
      namespace: a
"""
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.direct_apply(prev, new)
            self.assertIn(
                'id1', a.call_args[0][0]['dummies']['create'])
            self.assertEqual(
                len(a.call_args[0][0]['dummies']['update']), 0)
            self.assertEqual(
                len(a.call_args[0][0]['dummies']['delete']), 0)
            self.assertTrue(status)
        with patch('managesf.model.yamlbkd.engine.SFResourceBackendEngine.'
                   '_apply_changes') as a, \
                patch.dict(engine.MAPPING, {'dummies': Dummy},
                           clear=True):
            new = "a: True"
            eng = SFResourceBackendEngine(path, None)
            status, logs = eng.direct_apply(prev, new)
            self.assertFalse(status)
            self.assertListEqual(
                ['The main resource data structure is invalid'],
                logs)
            self.assertFalse(status)

    def test_get(self):
        with patch('managesf.model.yamlbkd.yamlbackend.'
                   'YAMLBackend.__init__') as i, \
                patch('managesf.model.yamlbkd.yamlbackend.'
                      'YAMLBackend.get_data') as g:
            i.return_value = None
            g.return_value = {}
            eng = SFResourceBackendEngine('/tmp/adir', None)
            data = eng.get('https://sftests.com/r/config', None)
            self.assertTrue(
                data.get('config-repo'), 'https://sftests.com/r/config')
            self.assertNotIn('connections', data['resources'])
            with patch('managesf.model.yamlbkd.engine.conf') as c:
                c.resources.get.return_value = {
                    'github.com': {'base_url': 'https://github.com'}}
                data = eng.get('https://sftests.com/r/config', None)
                self.assertTrue(
                    data.get('config-repo'), 'https://sftests.com/r/config')
                self.assertIn('connections', data['resources'])
                self.assertIn('github.com', data['resources']['connections'])

    def test_get_data_diff(self):
        with patch.dict(engine.MAPPING, {'dummies': Dummy}):
            # Test add resource change detected
            prev = {'resources': {'dummies': {}}}
            new = {'resources': {'dummies': {'myprojectid': {
                   'namespace': 'sf',
                   'name': 'myproject'},
            }}}
            eng = SFResourceBackendEngine(None, None)
            ret = eng._get_data_diff(prev, new)
            self.assertIn('dummies', ret)
            self.assertIn('create', ret['dummies'])
            self.assertIn('myprojectid', ret['dummies']['create'])
            self.assertDictEqual(new['resources']['dummies']['myprojectid'],
                                 ret['dummies']['create']['myprojectid'])
            self.assertEqual(len(ret['dummies']['delete'].keys()), 0)
            self.assertEqual(len(ret['dummies']['update'].keys()), 0)
            # Test delete resource change detected
            prev = {'resources': {'dummies': {'myprojectid': {
                    'namespace': 'sf',
                    'name': 'myproject'},
            }}}
            new = {'resources': {'dummies': {}}}
            eng = SFResourceBackendEngine(None, None)
            ret = eng._get_data_diff(prev, new)
            self.assertIn('myprojectid', ret['dummies']['delete'])
            self.assertEqual(len(ret['dummies']['create'].keys()), 0)
            self.assertEqual(len(ret['dummies']['update'].keys()), 0)
            # Test update resource change detected
            prev = {'resources': {'dummies': {'myprojectid': {
                    'namespace': 'sf'},
            }}}
            new = {'resources': {'dummies': {'myprojectid': {
                   'namespace': 'sf2'},
            }}}
            path = tempfile.mkdtemp()
            self.to_delete.append(path)
            eng = SFResourceBackendEngine(path, None)
            ret = eng._get_data_diff(prev, new)
            self.assertIn('myprojectid', ret['dummies']['update'])
            self.assertIn(
                'namespace',
                ret['dummies']['update']['myprojectid']['changed'])
            self.assertDictEqual(
                new['resources']['dummies']['myprojectid'],
                ret['dummies']['update']['myprojectid']['data'])
            # Test that multiple resource changes are detected
            prev = {'resources': {
                'dummies': {
                    'myprojectid': {
                        'namespace': 'sf',
                        'name': 'myproject'},
                    'superid': {
                        'namespace': 'super',
                        'name': 'project'}
                    },
                'groups': {}
            }}
            new = {'resources': {
                'dummies': {
                    'myprojectid': {
                        'namespace': 'sfnew',
                        'name': 'mynewproject'},
                    'myproject2id': {
                        'namespace': 'sfnew',
                        'name': 'newproject'}
                    },
                'groups': {
                    'mygroupid': {
                        'name': 'mynewgroup'},
                }
            }}
            eng = SFResourceBackendEngine(None, None)
            ret = eng._get_data_diff(prev, new)
            self.assertDictEqual(ret['dummies']['delete']['superid'],
                                 prev['resources']['dummies']['superid'])
            self.assertDictEqual(ret['dummies']['create']['myproject2id'],
                                 new['resources']['dummies']['myproject2id'])
            self.assertIn('namespace',
                          ret['dummies']['update']['myprojectid']['changed'])
            self.assertDictEqual(
                ret['dummies']['update']['myprojectid']['data'],
                new['resources']['dummies']['myprojectid'])
            self.assertDictEqual(ret['groups']['create']['mygroupid'],
                                 new['resources']['groups']['mygroupid'])
            # Test update resource change detected on a list
            prev = {'resources': {'dummies': {'myprojectid': {
                    'members': ['joe', 'paul']}}}}
            new = {'resources': {'dummies': {'myprojectid': {
                   'members': ['paul']}}}}
            path = tempfile.mkdtemp()
            self.to_delete.append(path)
            eng = SFResourceBackendEngine(path, None)
            ret = eng._get_data_diff(prev, new)
            self.assertSetEqual(
                ret['dummies']['update']['myprojectid']['changed'],
                set(['members']))

    def test_validate_changes(self):
        eng = SFResourceBackendEngine(None, None)
        validation_logs = []
        with patch.dict(engine.MAPPING, {'dummies': Dummy}):
            with patch.object(Dummy, 'validate') as v:
                changes = {'dummies': {'create': {'myprojectid': {}}}}
                eng._validate_changes(changes, validation_logs, {})
                self.assertTrue(v.called)
                v.reset_mock()
                changes = {'dummies': {'update': {
                    'myprojectid': {'data': {}, 'changed': []}}}}
                eng._validate_changes(changes, validation_logs, {})
                self.assertTrue(v.called)
                with patch.object(Dummy, 'is_mutable') as i:
                    v.reset_mock()
                    changes = {'dummies': {'update': {
                        'myprojectid': {'data': {}, 'changed': ['name']}}}}
                    eng._validate_changes(changes, validation_logs, {})
                    self.assertTrue(v.called)
                    self.assertTrue(i.called)
            # Be sure we have 3 validation msgs
            self.assertTrue(len(validation_logs), 3)
            validation_logs = []
            with patch.object(Dummy, 'validate') as v:
                v.side_effect = ResourceInvalidException('')
                changes = {'dummies': {'create': {'myprojectid': {}}}}
                self.assertRaises(ResourceInvalidException,
                                  eng._validate_changes,
                                  changes,
                                  validation_logs,
                                  {})
            with patch.object(Dummy, 'validate') as v:
                v.side_effect = ModelInvalidException('')
                changes = {'dummies': {'create': {'myprojectid': {}}}}
                self.assertRaises(ModelInvalidException,
                                  eng._validate_changes,
                                  changes,
                                  validation_logs,
                                  {})
            # Verify extra validations will be handled
            validation_logs = []
            with patch('managesf.model.yamlbkd.resources.'
                       'dummy.DummyOps.extra_validations') as xv:
                xv.return_value = ['error msg1', ' error msg2']
                changes = {'dummies': {'create': {'myprojectid': {
                    'namespace': 'sf', 'name': 'p1'}}}}
                self.assertRaises(ResourceInvalidException,
                                  eng._validate_changes,
                                  changes,
                                  validation_logs,
                                  {})
                self.assertTrue(xv.called)
                self.assertListEqual(['error msg1', ' error msg2'],
                                     validation_logs)

    def test_check_unicity_constraints(self):
        class Master(BaseResource):
            MODEL_TYPE = 'master'
            MODEL = {
                'name': (str, "+*", True, None, True, "desc"),
            }
            PRIORITY = 40
            PRIMARY_KEY = 'name'

        new = {
            'resources': {
                'masters': {
                    'm1': {
                        'name': 'ichiban',
                    },
                    'm2': {
                        'name': 'ichiban',
                    }
                }
            }
        }
        en = SFResourceBackendEngine(None, None)
        with patch.dict(engine.MAPPING,
                        {'masters': Master}):
            self.assertRaises(ResourceUnicityException,
                              en._check_unicity_constraints,
                              new)

    def test_check_deps_constraints(self):
        class Master(BaseResource):
            MODEL_TYPE = 'master'
            MODEL = {
                'name': (str, "+*", True, None, True, "desc"),
                'key1': (str, "+*", True, None, True, "desc"),
                'key2': (list, "+*", True, None, True, "desc"),
            }
            PRIORITY = 40
            PRIMARY_KEY = None

            def get_deps(self):
                deps = {'dummies': set([])}
                deps['dummies'].add(self.resource['key1'])
                for e in self.resource['key2']:
                    deps['dummies'].add(e)
                return deps

        new = {
            'resources': {
                'dummies': {
                    'd1': {
                        'name': 'dummy1',
                        'namespace': 'space',
                    },
                    'd2': {
                        'name': 'dummy2',
                        'namespace': 'space',
                    },
                    'd3': {
                        'name': 'dummy3',
                        'namespace': 'space',
                    },
                },
                'masters': {
                    'm1': {
                        'key1': 'd1',
                        'key2': ['d1', 'd2'],
                    }
                }
            }
        }

        en = SFResourceBackendEngine(None, None)
        with patch.dict(engine.MAPPING,
                        {'dummies': Dummy,
                         'masters': Master}):
            en._check_deps_constraints(new)
            # Add an unknown dependency
            new['resources']['masters']['m1']['key1'] = 'd4'
            self.assertRaises(ResourceDepsException,
                              en._check_deps_constraints,
                              new)

    def test_resolv_resources_need_refresh(self):
        class Master(BaseResource):
            MODEL_TYPE = 'master'
            MODEL = {
                'name': (str, "+*", True, None, True, "desc"),
                'key': (list, "+*", True, None, True, "desc"),
            }
            PRIORITY = 40
            PRIMARY_KEY = None

            def get_deps(self):
                deps = {'dummies': set([])}
                deps['dummies'].add(self.resource['key'])
                return deps

        # Engine dectected dummies:d1 has been updated
        changes = {'dummies': {'update': {'d1': {}}}}

        # masters:m1:key depends on dummies:d1
        tree = {
            'resources': {
                'dummies': {
                    'd1': {
                        'name': 'dummy1',
                        'namespace': 'space',
                    },
                },
                'masters': {
                    'm1': {
                        'key': 'd1',
                    }
                }
            }
        }

        en = SFResourceBackendEngine(None, None)
        with patch.dict(engine.MAPPING,
                        {'dummies': Dummy,
                         'masters': Master}):
            logs = en._resolv_resources_need_refresh(changes, tree)
            self.assertIn('m1', changes['masters']['update'])
            self.assertIn('d1', changes['dummies']['update'])
            self.assertEqual(len(changes['masters']['update']), 1)
            self.assertEqual(len(changes['dummies']['update']), 1)
            self.assertIn('Resource [type: masters, ID: m1] need a '
                          'refresh as at least one of its dependencies '
                          'has been updated', logs)
            self.assertEqual(len(logs), 1)

        # Engine dectected masters:m1 has been updated
        changes = {'masters': {'update': {'m1': {}}}}

        en = SFResourceBackendEngine(None, None)
        with patch.dict(engine.MAPPING,
                        {'dummies': Dummy,
                         'masters': Master}):
            logs = en._resolv_resources_need_refresh(changes, tree)
            # masters:m1 is on top of dependency chain
            # no addtionnal update trigger will be scheduled then
            self.assertEqual(len(logs), 0)
            self.assertIn('m1', changes['masters']['update'])
            self.assertEqual(len(changes['masters']['update']), 1)
            self.assertNotIn('dummies', changes)

        class Master2(BaseResource):
            MODEL_TYPE = 'master2'
            MODEL = {
                'name': (str, "+*", True, None, True, "desc"),
                'key': (str, "+*", True, None, True, "desc"),
            }
            PRIORITY = 30
            PRIMARY_KEY = None

            def get_deps(self):
                return {'masters': set([self.resource['key']])}

        # Engine dectected dummies:d1 has been updated
        changes = {'dummies': {'update': {'d1': {}}}}

        # masters:m1:key depends on dummies:d1
        # masters2:m1:key depends on master:m1
        tree = {
            'resources': {
                'dummies': {
                    'd1': {
                        'name': 'dummy1',
                        'namespace': 'space',
                    },
                },
                'masters': {
                    'm1': {
                        'key': 'd1',
                    }
                },
                'masters2': {
                    'm1': {
                        'key': 'm1',
                    }
                }
            }
        }
        en = SFResourceBackendEngine(None, None)
        with patch.dict(engine.MAPPING,
                        {'dummies': Dummy,
                         'masters': Master,
                         'masters2': Master2}):
            logs = en._resolv_resources_need_refresh(changes, tree)
            self.assertTrue(len(logs), 2)
            self.assertIn('Resource [type: masters, ID: m1] need a refresh '
                          'as at least one of its dependencies has been '
                          'updated', logs)
            self.assertIn('Resource [type: masters2, ID: m1] need a refresh '
                          'as at least one of its dependencies has been '
                          'updated', logs)
            self.assertIn('m1', changes['masters2']['update'])
            self.assertIn('m1', changes['masters']['update'])
            self.assertIn('d1', changes['dummies']['update'])
            self.assertTrue(len(changes['masters2']['update']), 1)
            self.assertTrue(len(changes['masters']['update']), 1)
            self.assertTrue(len(changes['dummies']['update']), 1)

    def test_apply_changes(self):
        eng = SFResourceBackendEngine(None, None)
        apply_logs = []
        with patch.dict(engine.MAPPING, {'dummies': Dummy}):
            with patch('managesf.model.yamlbkd.resources.'
                       'dummy.DummyOps.create') as c:
                c.return_value = []
                changes = {'dummies': {'create': {'myprojectid': {}}}}
                self.assertFalse(eng._apply_changes(changes, apply_logs, {}))
                self.assertTrue(c.called)
            self.assertIn(
                'Resource [type: dummies, ID: myprojectid] '
                'will be created.',
                apply_logs)
            self.assertIn(
                'Resource [type: dummies, ID: myprojectid] '
                'has been created.',
                apply_logs)
            self.assertTrue(len(apply_logs), 2)

            apply_logs = []
            with patch('managesf.model.yamlbkd.resources.'
                       'dummy.DummyOps.create') as c:
                c.return_value = ["Resource API error"]
                changes = {'dummies': {'create': {'myprojectid': {}}}}
                self.assertTrue(eng._apply_changes(changes, apply_logs, {}))

            apply_logs = []
            with patch('managesf.model.yamlbkd.resources.'
                       'dummy.DummyOps.create') as c:
                c.return_value = ["Resource API error"]
                changes = {
                    'dummies': {
                        'create': {
                            'myprojectid': {}
                        },
                        'update': {
                            'myprojectid2': {
                                'data': {'key': 'value'},
                                'changed': ['key']
                            }
                        }
                    }
                }
                self.assertTrue(eng._apply_changes(changes, apply_logs, {}))
                self.assertIn('Resource [type: dummies, ID: myprojectid] '
                              'will be created.',
                              apply_logs)
                self.assertIn('Resource API error',
                              apply_logs)
                self.assertIn('Resource [type: dummies, ID: myprojectid] '
                              'create op failed.',
                              apply_logs)
                self.assertIn('Resource [type: dummies, ID: myprojectid2] '
                              'will be updated.',
                              apply_logs)
                self.assertIn('Resource [type: dummies, ID: myprojectid2] '
                              'has been updated.',
                              apply_logs)

            # Verify an unexpected exception is properly catched
            apply_logs = []
            with patch('managesf.model.yamlbkd.resources.'
                       'dummy.DummyOps.create') as c:
                c.side_effect = Exception('Random Error msg')
                changes = {'dummies': {'create': {'myprojectid': {}}}}
                self.assertTrue(eng._apply_changes(changes, apply_logs, {}))
                self.assertIn('Resource [type: dummies, ID: myprojectid] '
                              'create op error (Random Error msg).',
                              apply_logs)

            # Verify an unexpected exception does not exit _apply_changes
            apply_logs = []
            with patch('managesf.model.yamlbkd.resources.'
                       'dummy.DummyOps.create') as c:
                c.side_effect = Exception('Random Error msg')
                changes = {
                    'dummies': {
                        'create': {
                            'myprojectid1': {},
                            'myprojectid2': {},
                            'myprojectid3': {},
                        },
                    }
                }
                self.assertTrue(eng._apply_changes(changes, apply_logs, {}))
                for r in ('myprojectid1', 'myprojectid2', 'myprojectid3'):
                    self.assertIn(
                        'Resource [type: dummies, ID: %s] will be created.' % (
                            r), apply_logs)
                    self.assertIn(
                        'Resource [type: dummies, ID: %s] create op error '
                        '(Random Error msg).' % r, apply_logs)
                    self.assertIn(
                        'Resource [type: dummies, ID: %s] create op '
                        'failed.' % r, apply_logs)

    def test_get_missing_resources(self):
        class Dummy2(Dummy):
            MODEL_TYPE = 'dummy2'
            MODEL = {
                'name': (
                    str,
                    '.*',
                    True,
                    None,
                    False,
                    "Resource name",
                ),
                'deps': (
                    list,
                    '.*',
                    False,
                    [],
                    True,
                    "Resource dependencies",
                ),
            }
            PRIMARY_KEY = 'name'

            def get_deps(self, keyname=False):
                if keyname:
                    return 'deps'
                else:
                    return {'dummies': set(self.resource['deps'])}

        with patch('managesf.model.yamlbkd.yamlbackend.'
                   'YAMLBackend.__init__'), \
                patch.object(SFResourceBackendEngine, 'get') as g, \
                patch.dict(engine.MAPPING,
                           {'dummies': Dummy, 'dummies2': Dummy2},
                           clear=True), \
                patch('managesf.model.yamlbkd.resources.'
                      'dummy.DummyOps.get_all') as ga:
            eng = SFResourceBackendEngine(None, None)
            # PRIMARY_KEY of Dummy is 'name'
            # PRIMARY_KEY of Dummy2 is 'name'

            # Check basic scenario. dummies:d2 is really new
            current_resources = {
                'dummies': {
                    'd1': {
                        'namespace': 'sf',
                        'name': 'd1',
                    },
                }
            }
            real_resources = {
                'dummies': {
                    'd2': {
                        'namespace': 'sf',
                        'name': 'd2',
                    },
                }
            }
            g.return_value = {'resources': current_resources}
            ga.return_value = ([], real_resources)
            logs, ret = eng.get_missing_resources(None, None)
            expected = {
                'resources': {
                    'dummies': {
                        'd2': {
                            'namespace': 'sf',
                        },
                    }
                }
            }
            self.assertDictEqual(ret, expected)
            self.assertListEqual(logs, [])

            # Check both resources are detected similar.
            # dummies:d1 is dummies:dummy-d1-id
            current_resources = {
                'dummies': {
                    'd1': {
                        'namespace': 'sf',
                    },
                }
            }
            real_resources = {
                'dummies': {
                    'd1': {
                        'namespace': 'sf',
                    },
                }
            }
            g.return_value = {'resources': current_resources}
            ga.return_value = ([], real_resources)
            logs, ret = eng.get_missing_resources(None, None)
            expected = {
                'resources': {}
            }
            self.assertDictEqual(ret, expected)
            self.assertListEqual(logs, [])

            # Check both resources are detected similar.
            # dummies:d1 is dummies:dummy-d1-id
            # dummies2:d2_1 depends on dummies:d1 but
            # dummies:d1 is know under dummies:dummy-d1-id
            # This check make sure the deps if is updated.
            current_resources = {
                'dummies': {
                    'd1': {
                        'namespace': 'sf',
                    },
                }
            }
            real_resources = {
                'dummies': {
                    'd1': {
                        'namespace': 'sf',
                    },
                },
                'dummies2': {
                    'd2_1': {
                        'deps': ['d1'],
                    },
                },
            }
            g.return_value = {'resources': current_resources}
            ga.return_value = ([], real_resources)
            logs, ret = eng.get_missing_resources(None, None)
            expected = {
                'resources': {
                    'dummies2': {
                        'd2_1': {
                            'deps': ['d1'],
                        }
                    }
                }
            }
            self.assertDictEqual(ret, expected)
            self.assertListEqual(logs, [])



#!/usr/bin/env python
#
# Copyright (C) 2015 Red Hat <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import logging

from managesf.services import base


logger = logging.getLogger(__name__)


class SFGerritProjectManager(base.ProjectManager):

    def get_user_groups(self, user):
        client = self.plugin.get_client()
        groups = client.get_user_groups(user)
        if isinstance(groups, bool):
            logger.info(u"[%s] Could not find user groups %s: %s" % (
                self.plugin.service_name, user, str(groups)))
            groups = []
        return groups



#
# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from managesf.model.yamlbkd.resource import BaseResource


logger = logging.getLogger(__name__)


class ProjectOps(object):

    def __init__(self, conf, new):
        self.conf = conf
        self.new = new
        self.client = None

    def create(self, **kwargs):
        logs = []
        return logs

    def update(self, **kwargs):
        logs = []
        return logs

    def delete(self, **kwargs):
        logs = []
        return logs

    def extra_validations(self, **kwargs):
        logs = []
        name = kwargs['name']

        project = self.new['resources']['projects'][name]
        for sr in project['source-repositories']:
            if isinstance(sr, dict):
                sr_name = list(sr.keys())[0]
                sr_attrs = sr[sr_name]
                private = sr_attrs.get('private', False)
            else:
                sr_name = sr
                private = False
            repo = self.new['resources'].get('repos', {}).get(sr_name)
            if not repo:
                continue
            acl = self.new['resources']['acls'][repo['acl']]
            if ('read = deny group Registered Users' in acl['file'] and
                    'read = deny group Anonymous Users' in acl['file']):
                if not private:
                    logs.append(
                        "%s repository use a private Gerrit ACL but are not"
                        " defined as private in the project %s" % (
                            sr_name, name))

        return logs


class Project(BaseResource):

    DESCRIPTION = ("The project resource can be is used to describe a "
                   "project. It can be seen as the top level resource type in "
                   "in this model. You can use it reference multiple Git "
                   "repositories and multiple link to external resources like "
                   "a project website and the issues tracker website.")

    MODEL_TYPE = 'project'
    MODEL = {
        'name': (
            str,
            r'^([a-zA-Z0-9\-_\./])+$',
            False,
            "",
            False,
            "The project name",
        ),
        'description': (
            str,
            '.*',
            True,
            None,
            True,
            "The project description",
        ),
        'tenant': (
            str,
            '.*',
            False,
            "",
            True,
            "The tenant name",
        ),
        'connection': (
            str,
            '.*',
            False,
            "",
            True,
            "The default connection for source-repositories",
        ),
        'website': (
            str,
            '.*',
            False,
            "",
            True,
            "The project web page link",
        ),
        'documentation': (
            str,
            '.*',
            False,
            "",
            True,
            "The project documentation link",
        ),
        'issue-tracker-url': (
            str,
            '.*',
            False,
            "",
            True,
            "The project issue tracker link",
        ),
        'review-dashboard': (
            str,
            r'^([a-zA-Z0-9\-_])*$',
            False,
            "",
            True,
            "A gerrit dashboard name reference",
        ),
        'mailing-lists': (
            list,
            '.+@.+',
            False,
            [],
            True,
            "Email addresses of project mailing lists",
        ),
        'contacts': (
            list,
            '.+@.+',
            False,
            [],
            True,
            "Email addresses of project main contacts",
        ),
        'source-repositories': (
            list,
            (dict, '.+'),
            True,
            None,
            True,
            "Code source repositories related to the project",
        ),
        'options': (
            list,
            '.+',
            False,
            [],
            True,
            "Project options for services",
        ),
    }
    PRIORITY = 10
    PRIMARY_KEY = 'name'
    CALLBACKS = {
        'update': lambda conf, new, kwargs:
            ProjectOps(conf, new).update(**kwargs),
        'create': lambda conf, new, kwargs:
            ProjectOps(conf, new).create(**kwargs),
        'delete': lambda conf, new, kwargs:
            ProjectOps(conf, new).delete(**kwargs),
        'extra_validations': lambda conf, new, kwargs:
            ProjectOps(conf, new).extra_validations(**kwargs),
        'get_all': lambda conf, new: ([], {}),
    }

    def get_deps(self, keyname=False):
        if keyname:
            return 'source-repositories'
        ids = set()
        for sr in self.resource['source-repositories']:
            if isinstance(sr, dict):
                ids.add(list(sr)[0])
            else:
                ids.add(sr)
        return {'repos': ids}

    def is_deps_soft(self):
        return True

    def transform_for_get(self):
        ret = {}
        for key, data in self.resource.items():
            if key == 'source-repositories':
                srs = []
                for sr in data:
                    if isinstance(sr, (str, bytes)):
                        srs.append({sr: {}})
                    else:
                        srs.append(sr)
            else:
                ret[key] = data
        ret['source-repositories'] = srs
        return ret



# Copyright (C) 2017 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import yaml
import shutil
from unittest import TestCase

from managesf.tests import dummy_conf
from managesf.tests.fixtures import SAMPLE_RESOURCES_TREE
from managesf.tests import resources_test_utils as rtu

from webtest import TestApp
from pecan import load_app
from mock import patch

from managesf.model.yamlbkd.resources.dummy import Dummy

c = dummy_conf()
config = {'services': c.services,
          'gerrit': c.gerrit,
          'app': c.app,
          'admin': c.admin,
          'sqlalchemy': c.sqlalchemy,
          'managesf': c.managesf,
          'policy': c.policy,
          'resources': c.resources,
          'api': c.api, }

# App must be loaded before we can import v2 managers
TestApp(load_app(config))


from managesf.api.v2.resources.services import manageSF  # noQA


class BaseTestResourceEndpoint(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.db_path = []
        repo_path = rtu.prepare_git_repo(cls.db_path)
        rtu.add_yaml_data(repo_path, SAMPLE_RESOURCES_TREE)
        # Init the YAML DB
        rtu.prepare_db_env(cls.db_path)
        c.resources['master_repo'] = 'file://%s' % repo_path
        cls.manager = manageSF.SFResourcesManager(c)

    @classmethod
    def tearDownClass(cls):
        for db_path in cls.db_path:
            if os.path.isdir(db_path):
                shutil.rmtree(db_path)
            else:
                os.unlink(db_path)


class TestResourcesManager(BaseTestResourceEndpoint):

    @classmethod
    def setUpClass(cls):
        cls.db_path = []

    def prepare_repo(self, data):
        repo_path = rtu.prepare_git_repo(self.db_path)
        rtu.add_yaml_data(repo_path, data)
        # Init the YAML DB
        rtu.prepare_db_env(self.db_path)
        return repo_path

    def test_get(self):
        """test raw resources get on resources manager"""
        data = {'resources': {'dummies': {
                'id1': {'name': 'resource_a'}
                }}}
        repo_path = self.prepare_repo(data)
        c.resources['master_repo'] = 'file://%s' % repo_path
        manager = manageSF.SFResourcesManager(c)
        with patch.dict('managesf.model.yamlbkd.engine.MAPPING',
                        {'dummies': Dummy}):
            ret = manager.resources.get()
        self.assertIn("resources", ret)
        self.assertEqual(
            ret.get("config-repo"), 'file://%s' % repo_path)

    def test_create(self):
        """test validate resources on the resources manager"""
        data = {'resources': {'dummies': {}}}
        repo_path = self.prepare_repo(data)
        proposed_data = {
            'resources': {
                'dummies': {
                    'id1': {
                        'namespace': 'awesome',
                        'name': 'p1'}
                }
            }
        }
        c.resources['master_repo'] = 'file://%s' % repo_path
        manager = manageSF.SFResourcesManager(c)
        kwargs = {
            'data': {
                'fakepath': yaml.dump(proposed_data,
                                      default_flow_style=False)
            }
        }
        with patch.dict('managesf.model.yamlbkd.engine.MAPPING',
                        {'dummies': Dummy}):
            status, log = manager.resources.create(**kwargs)
        self.assertTrue(status)
        self.assertIn(
            'Resource [type: dummies, ID: id1] is going '
            'to be created.', log)
        proposed_data = {
            'resources': {
                'dummies': {
                    'idbogus': {
                        'namespace': 'awesome',
                        'n4me': 'p3'},
                    'id2': {
                        'namespace': 'awesome',
                        'name': 'p2'}
                }
            }
        }
        kwargs = {
            'data': {
                'fakepath': yaml.dump(proposed_data,
                                      default_flow_style=False)
            }
        }
        with patch.dict('managesf.model.yamlbkd.engine.MAPPING',
                        {'dummies': Dummy}):
            status, log = manager.resources.create(**kwargs)
        self.assertFalse(status)
        self.assertIn(
            "Resource [type: dummy, ID: idbogus] contains extra keys. "
            "Please check the model.", log)



# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from managesf.services.gerrit import SoftwareFactoryGerrit
from managesf.services.gerrit import utils
from managesf.model.yamlbkd.resource import BaseResource

# ## DEBUG statements to ease run that standalone ###
# import logging
# logging.basicConfig()
# logging.getLogger().setLevel(logging.DEBUG)
# requests_log = logging.getLogger("requests.packages.urllib3")
# requests_log.setLevel(logging.DEBUG)
# requests_log.propagate = True
#
# from pecan import configuration
# from managesf.model.yamlbkd.resources.group import GroupOps
# conf = configuration.conf_from_file('/var/www/managesf/config.py')
# g = GroupOps(conf, {})
# g._set_client()
# ###


logger = logging.getLogger(__name__)
UNMANAGED_GERRIT_GROUPS = ('Administrators',
                           'Service Users')
DELETED_GROUP_RENAME_PATTERN = "_deleted_group_%s"


class GroupOps(object):

    def __init__(self, conf, new):
        self.conf = conf
        self.new = new

    def _set_client(self):
        gerrit = SoftwareFactoryGerrit(self.conf)
        self.client = gerrit.get_client()

    def group_update_description(self, name, description):
        data = {"description": description}
        return self.client.put("groups/%s/description" % name, data)

    def get_all(self):
        logs = []
        groups = {}

        self._set_client()

        try:
            all_groups = self.client.get_groups()
        except Exception as e:
            logger.exception("get_groups failed")
            logs.append("Group list: err API returned %s" % e)
            return logs, groups
        groups = {}
        for gname, data in all_groups.items():
            if gname in UNMANAGED_GERRIT_GROUPS:
                continue
            groups[gname] = {}
            groups[gname]['name'] = gname
            groups[gname]['description'] = data.get('description', '')
            groups[gname]['members'] = []
            try:
                members = self.client.get_group_members(str(data['group_id']))
                groups[gname]['members'] = [
                    m['email'] for m in members if 'email' in m.keys()]
            except Exception as e:
                logger.exception("get_group_members failed")
                logs.append(
                    "Group list members [%s]: err API "
                    "returned %s" % (gname, e))
        return logs, {'groups': groups}

    def create(self, **kwargs):
        logs = []
        name = kwargs['name']
        members = kwargs['members']
        description = kwargs['description']

        self._set_client()

        try:
            self.client.create_group(name, description)
        except Exception as e:
            logger.exception("create_group failed")
            logs.append("Group create: err API returned %s" % e)

        # Remove auto added admin
        try:
            self.client.delete_group_member(name, "admin")
        except utils.NotFound:
            pass
        except Exception as e:
            logger.exception("delete_group_member failed")
            logs.append("Group create [del member: admin]: "
                        "err API returned %s" % e)

        if members:
            for member in members:
                try:
                    self.client.add_group_member(member, name)
                except Exception as e:
                    logger.exception("add_group_member failed")
                    logs.append("Group create [add member: %s]: "
                                "err API returned %s" % (member, e))

        return logs

    def delete(self, **kwargs):
        logs = []
        name = kwargs['name']

        self._set_client()

        # Remove all group members to avoid left overs in the DB
        gid = self.client.get_group_id(name)
        current_members = [u['email'] for u in
                           self.client.get_group_members(gid)
                           if 'email' in u.keys()]
        for member in current_members:
            try:
                self.client.delete_group_member(name, member)
            except Exception as e:
                logger.exception("delete_group_member failed")
                logs.append("Group delete [del member: %s]: "
                            "err API returned %s" % (member, e))

        # Remove all included groups members to avoid left overs in the DB
        grps = [g['name'] for
                g in self.client.get_group_group_members(gid)]
        for grp in grps:
            try:
                self.client.delete_group_group_member(gid, grp)
            except Exception as e:
                logger.exception("delete_group_group_member failed")
                logs.append("Group delete [del included group %s]: "
                            "err API returned %s" % (grp, e))

        # Rename the group / Gerrit does not provide an API to delete a
        # group. Instead we rename it.
        try:
            self.client.rename_group(
                gid, DELETED_GROUP_RENAME_PATTERN % name)
        except Exception as e:
            logger.exception("rename_group failed")
            logs.append("Group delete [rename deleted group: %s]: "
                        "err API returned %s" % (name, e))

        return logs

    def update(self, **kwargs):
        logs = []
        name = kwargs['name']
        members = kwargs['members']
        description = kwargs['description']

        self._set_client()

        gid = self.client.get_group_id(name)
        current_members = [u['email'] for u in
                           self.client.get_group_members(gid) if
                           'email' in u.keys()]
        to_add = set(members) - set(current_members)
        to_del = set(current_members) - set(members)

        for mb in to_del:
            try:
                self.client.delete_group_member(name, mb)
            except Exception as e:
                logger.exception("delete_group_member failed")
                logs.append("Group update [del member: %s]: "
                            "err API returned %s" % (mb, e))

        for mb in to_add:
            try:
                self.client.add_group_member(mb, name)
            except Exception as e:
                logger.exception("add_group_member failed")
                logs.append("Group update [add member: %s]: "
                            "err API returned %s" % (mb, e))

        try:
            self.group_update_description(name, description)
        except Exception as e:
            logger.exception("group_update_description failed")
            logs.append("Group update [update description]: "
                        "err API returned %s" % e)

        # Remove included groups if exist ! We are not supporting that
        grps = [g['name'] for
                g in self.client.get_group_group_members(gid)]
        for grp in grps:
            try:
                self.client.delete_group_group_member(gid, grp)
            except Exception as e:
                logger.exception("delete_group_group_member failed")
                logs.append("Group update [del included group %s]: "
                            "err API returned %s" % (grp, e))

        return logs

    def check_account_members(self, members):
        logs = []

        self._set_client()

        for member in members:
            try:
                self.client.get_account(member)
            except utils.NotFound:
                logs.append("Check group members [%s does not exists]: "
                            "err API unable to find the member" % member)
        return logs

    def extra_validations(self, **kwargs):
        """ This checks that requested members exists
        inside the backend.
        """
        logs = []
        members = kwargs['members']
        name = kwargs['name']

        # Return log msgs making the validation fail in the engine
        if name in UNMANAGED_GERRIT_GROUPS:
            logs.append("Check group name [%s in not managed by this API]" % (
                        name))

        logs.extend(self.check_account_members(members))

        return logs


class Group(BaseResource):

    MODEL_TYPE = 'group'
    DESCRIPTION = ("The group resource is used to define a group of "
                   "known user on the platform. Users must be referenced "
                   "by their email address. A group can be share between "
                   "multiple acls.")
    MODEL = {
        'name': (
            str,
            r'^([a-zA-Z0-9\-_\./])+$',
            False,
            "",
            False,
            "The group name",
        ),
        'description': (
            str,
            '.*',
            False,
            "",
            True,
            "The group description",
        ),
        'members': (
            list,
            '.+@.+',
            False,
            [],
            True,
            "The group members list",
        ),
    }
    PRIORITY = 40
    PRIMARY_KEY = 'name'
    CALLBACKS = {
        'update': lambda conf, new, kwargs:
            GroupOps(conf, new).update(**kwargs),
        'create': lambda conf, new, kwargs:
            GroupOps(conf, new).create(**kwargs),
        'delete': lambda conf, new, kwargs:
            GroupOps(conf, new).delete(**kwargs),
        'extra_validations': lambda conf, new, kwargs:
            GroupOps(conf, new).extra_validations(**kwargs),
        'get_all': lambda conf, new:
            GroupOps(conf, new).get_all(),
    }



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from managesf.model.yamlbkd.resource import BaseResource

# This is a Test object


class DummyOps(object):

    def __init__(self, conf, new):
        self.conf = conf
        self.new = new

    def create(self, **kwargs):
        return []

    def delete(self, **kwargs):
        return []

    def update(self, **kwargs):
        return []

    def extra_validations(self, **kwargs):
        return []

    def get_all(self):
        return [], {}


class Dummy(BaseResource):
    MODEL_TYPE = 'dummy'
    MODEL = {
        'namespace': (
            str,
            r'^([a-zA-Z0-9\-_])+$',
            True,
            None,
            False,
            "",
        ),
        'name': (
            str,
            r'^([a-zA-Z0-9\-_])+$',
            False,
            "",
            False,
            "",
        ),
        'description': (
            str,
            r'^([a-zA-Z0-9\-_ ])*$',
            False,
            "",
            True,
            "",
        ),
    }
    PRIMARY_KEY = 'name'
    PRIORITY = 50
    CALLBACKS = {
        'update': lambda conf, new, kwargs:
            DummyOps(conf, new).update(**kwargs),
        'create': lambda conf, new, kwargs:
            DummyOps(conf, new).create(**kwargs),
        'delete': lambda conf, new, kwargs:
            DummyOps(conf, new).delete(**kwargs),
        'extra_validations': lambda conf, new, kwargs:
            DummyOps(conf, new).extra_validations(**kwargs),
        'get_all': lambda conf, new:
            DummyOps(conf, new).get_all(),
    }



#
# Copyright (C) 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from managesf.lib.auth.cauth import CAuthAuthenticator
from managesf.lib.auth.keycloak import KeycloakAuthenticator


log = logging.getLogger(__name__)


def get_auth_engine(services):
    try:
        if 'keycloak' in services:
            return KeycloakAuthenticator()
        else:
            return CAuthAuthenticator()
    except Exception as e:
        log.info(
            'An exception occurred while getting '
            'the authentication engine: "%s"; defaulting to cauth' % e)
        return CAuthAuthenticator()



#
# Copyright (C) 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import requests


def _get(url, verify=True):
    resp = requests.get(url, allow_redirects=False,
                        verify=verify)
    return resp.json()


def get_cauth_info(auth_server, verify=True):
    url = "%s/auth/about/" % auth_server
    return _get(url, verify)


def get_managesf_info(gateway_url, verify=True):
    url = "%s/manage/about/" % gateway_url
    return _get(url, verify).get('service', {}).get('services', [])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import TestCase

from managesf.model.yamlbkd.resource import BaseResource
from managesf.model.yamlbkd.resource import ResourceInvalidException
from managesf.model.yamlbkd.resource import ModelInvalidException


class ResourcesTest(TestCase):

    def test_resource_model(self):
        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'key': (str, ".*", False, "string", True, "desc"),
            }
            PRIMARY_KEY = None
            PRIORITY = 10

        self.assertRaises(ModelInvalidException,
                          R1, 'id', {})

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, ".*", False, "string", True, "Name"),
                'key': (str, ".*", False, "string", True, "desc"),
            }
            PRIMARY_KEY = None
            PRIORITY = 10

        r = R1('id', {})
        self.assertEqual(r.resource['name'], 'id')
        r = R1('id', {'name': 'overwritten'})
        self.assertEqual(r.resource['name'], 'id')

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, "[0-9]+", False, "123", True, "Name"),
            }
            PRIMARY_KEY = None
            PRIORITY = 10

        r = R1('id', {})
        self.assertRaises(ResourceInvalidException,
                          r.validate)

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, ".*", False, "string", True, "Name"),
                'key': (int, None, False, "string", True, "desc"),
            }
            PRIMARY_KEY = 'name'
            PRIORITY = 10

        self.assertRaises(ModelInvalidException,
                          R1, 'id', {})

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, ".*", False, "string", True, "Name"),
                'key': (int, None, False, "string", True, "desc"),
            }
            PRIMARY_KEY = None
            PRIORITY = 10

        self.assertRaises(ModelInvalidException,
                          R1, 'id', {})

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'key': (str, '^[A-Z]+$', False, "123", True, "desc"),
                'name': (str, ".*", True, None, False, "desc"),
            }
            PRIMARY_KEY = 'name'
            PRIORITY = 10
        self.assertRaises(ModelInvalidException,
                          R1, 'id', {'name': '123'})

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, ".*", False, "string", True, "Name"),
                'key': (dict, ('.*', '.*'), False, "data", True, "desc"),
            }
            PRIMARY_KEY = 'name'
            PRIORITY = 10
        self.assertRaises(ModelInvalidException,
                          R1, 'id', {'key': {}})

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, ".*", False, "string", True, "Name"),
                'key': (dict, '.*', False, {"key": "value"}, True, "desc"),
            }
            PRIMARY_KEY = 'name'
            PRIORITY = 10
        self.assertRaises(ModelInvalidException,
                          R1, 'id', {'key': {}})

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, ".*", False, "string", True, "Name"),
                'key': (int, None, True, None, True, "desc"),
                'key2': (str, ".+", False, "default", True, "desc"),
                'key3': (list, "[0-9]+", False, [], True, "desc"),
                'key4': (dict, ('[a-z]', r'\d+'), False, {}, True, "desc"),
                'key5': (list, (dict, '.+'), False, [], True, "desc"),
            }
            PRIMARY_KEY = None
            PRIORITY = 10

        res = R1('id', {'key': 1})
        res.validate()
        res.set_defaults()
        resource = res.get_resource()
        self.assertIn('key2', resource)
        self.assertTrue(isinstance(resource['key2'],
                                   str))
        res = R1('id', {'key': 'string'})
        self.assertRaises(ResourceInvalidException,
                          res.validate)
        res = R1('id', {})
        self.assertRaises(ResourceInvalidException,
                          res.validate)
        res = R1('id', {'key': 1, 'extra': 'value'})
        self.assertRaises(ResourceInvalidException,
                          res.validate)
        res = R1('id', {'key': 1,
                        'key3': ['12', '13']})
        res.validate()
        res = R1('id', {'key': 1,
                        'key3': ['12', 'abc']})
        self.assertRaises(ResourceInvalidException,
                          res.validate)
        res = R1('id', {'key': 1,
                        'key3': [
                            '12',
                            {'a1': None}]})
        self.assertRaises(ResourceInvalidException,
                          res.validate)
        res = R1('id', {'key': 1,
                        'key4': {}})
        res.validate()
        res = R1('id', {'key': 1,
                        'key4': {'master': '1234',
                                 'dev': '1234'}})
        res = R1('id', {'key': 1,
                        'key4': {'master': '1234',
                                 'dev': 'abc'}})
        self.assertRaises(ResourceInvalidException,
                          res.validate)
        res = R1('id', {'key': 1,
                        'key5': [
                            'a1',
                            {'a2': None}
                        ]})
        res.validate()
        res = R1('id', {'key': 1,
                        'key5': [
                            'a1',
                            {'a2': None,
                             'morekey': None}
                        ]})
        self.assertRaises(ResourceInvalidException,
                          res.validate)
        res = R1('id', {'key': 1,
                        'key4': {
                            'subk1': '123',
                            'subk2': True,
                            'subk3': 123
                            }})
        res.validate()

    def test_resource_model_callbacks(self):
        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, ".*", False, "string", True, "Name"),
            }
            PRIORITY = 10
            PRIMARY_KEY = None
            CALLBACKS = {
                'update': NotImplementedError,
                'create': lambda: None,
                'delete': lambda: None,
                'extra_validations': NotImplementedError,
                'get_all': NotImplementedError,
            }
        R1('id', {})

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, ".*", False, "string", True, "Name"),
            }
            PRIORITY = 10
            PRIMARY_KEY = None
            CALLBACKS = {
                'delete': lambda: None,
            }
        self.assertRaises(ModelInvalidException,
                          R1, 'id', {})

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, ".*", False, "string", True, "Name"),
            }
            PRIMARY_KEY = None
            PRIORITY = 10
            CALLBACKS = {
                'update': NotImplementedError,
                'create': None,
                'delete': lambda: None,
                'extra_validations': NotImplementedError,
                'get_all': NotImplementedError,
            }
        self.assertRaises(ModelInvalidException,
                          R1, 'id', {})

        class Ops(object):
            def create(self, **kwargs):
                return kwargs

        class R1(BaseResource):
            MODEL_TYPE = 'test'
            MODEL = {
                'name': (str, ".*", False, "string", True, "Name"),
            }
            PRIMARY_KEY = None
            PRIORITY = 10
            CALLBACKS = {
                'update': NotImplementedError,
                'create': lambda kwargs: Ops().create(**kwargs),
                'delete': lambda: None,
                'extra_validations': NotImplementedError,
                'get_all': NotImplementedError,
            }
        res = R1('id', {})
        self.assertDictEqual({'arg': 'value'},
                             res.CALLBACKS['create']({'arg': 'value'}))



# Copyright (C) 2015 Red Hat <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import logging

from requests.auth import HTTPBasicAuth

from managesf.services import base
from managesf.services.gerrit import project
from managesf.services.gerrit import user
from managesf.services.gerrit import review
from managesf.services.gerrit import group
from managesf.services.gerrit.utils import GerritClient

logger = logging.getLogger(__name__)


class Gerrit(base.BaseCodeReviewServicePlugin):
    """Plugin managing the Gerrit Code Review service."""

    _config_section = "gerrit"
    service_name = "gerrit"

    def __init__(self, conf):
        super(Gerrit, self).__init__(conf)
        self.project = None
        self.user = None
        self.review = None
        self.group = None

    def get_client(self, cookie=None):
        raise NotImplementedError


ADMIN_COOKIE = None
ADMIN_COOKIE_DATE = 0
COOKIE_VALIDITY = 60


class SoftwareFactoryGerrit(Gerrit):
    """"""

    def __init__(self, conf):
        super(SoftwareFactoryGerrit, self).__init__(conf)
        self.project = project.SFGerritProjectManager(self)
        self.user = user.SFGerritUserManager(self)
        self.review = review.SFGerritReviewManager(self)
        self.group = group.SFGerritGroupManager(self)

    def get_client(self, cookie=None):
        auth = HTTPBasicAuth("admin", self.conf['password'])
        return GerritClient(self.conf['url'], auth)



#!/usr/bin/env python
#
# Copyright (C) 2014 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import requests


class IntrospectionNotAvailableError(Exception):
    pass


def _get_service_info(url, verify=True):
    resp = requests.get(url, allow_redirects=False,
                        verify=verify)
    return resp.json()


def get_managesf_info(auth_server, verify=True):
    url = "%s/about/" % auth_server
    return _get_service_info(url, verify)



#!/usr/bin/env python
#
# Copyright (C) 2014 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from pecan import conf
from pecan import expose
from pecan.rest import RestController
import pkg_resources


class IntrospectionController(RestController):
    """A controller that allows a client to know more about the server."""

    def get_managesf_version(self):
        managesf = pkg_resources.get_distribution('managesf')
        return managesf.version

    @expose(template='json')
    def index(self, **kwargs):
        return_value = {'service': {
            'name': 'managesf',
            'version': self.get_managesf_version(),
            # TODO(mhu) this should not be hardcoded. Wait for plugins!
            'services': conf.services, }}
        return return_value



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import TestCase

from mock import patch, call

from managesf.tests import dummy_conf
from managesf.model.yamlbkd.resources.group import GroupOps
from managesf.services.gerrit import utils


class GroupOpsTest(TestCase):
    @classmethod
    def setupClass(cls):
        cls.conf = dummy_conf()

    def test_create(self):
        o = GroupOps(self.conf, None)

        kwargs = {'name': 'space/g1',
                  'description': 'A description',
                  'members': ['body@sftests.com', 'body2@sftests.com']}
        with patch('managesf.services.gerrit.utils.'
                   'GerritClient.create_group') as cg, \
                patch('managesf.services.gerrit.utils.'
                      'GerritClient.add_group_member') as agm, \
                patch('managesf.services.gerrit.utils.'
                      'GerritClient.delete_group_member') as dgm:
            logs = o.create(**kwargs)
            self.assertEqual(len(cg.call_args_list), 1)
            self.assertEqual(cg.call_args_list[0],
                             call('space/g1', 'A description'))
            self.assertEqual(len(agm.call_args_list), 2)
            self.assertEqual(agm.call_args_list[0],
                             call('body@sftests.com', 'space/g1'))
            self.assertEqual(agm.call_args_list[1],
                             call('body2@sftests.com', 'space/g1'))
            self.assertEqual(len(dgm.call_args_list), 1)
            self.assertEqual(dgm.call_args_list[0],
                             call('space/g1', "admin"))
            self.assertEqual(len(logs), 0)
        with patch('managesf.services.gerrit.utils.'
                   'GerritClient.create_group') as cg, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'add_group_member') as agm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'delete_group_member') as dgm:
            cg.side_effect = Exception('Random error')
            agm.side_effect = Exception("HTTP 404/409")
            dgm.side_effect = Exception("HTTP 404/409")
            logs = o.create(**kwargs)
            self.assertIn('Group create: err API returned Random error',
                          logs)
            self.assertIn('Group create [add member: body@sftests.com]: '
                          'err API returned HTTP 404/409', logs)
            self.assertIn('Group create [add member: body2@sftests.com]: '
                          'err API returned HTTP 404/409', logs)
            self.assertIn('Group create [del member: admin]: '
                          'err API returned HTTP 404/409', logs)
            self.assertEqual(len(logs), 4)

    def test_delete(self):
        o = GroupOps(self.conf, None)

        kwargs = {'name': 'space/g1'}

        with patch('managesf.services.gerrit.utils.'
                   'GerritClient.get_group_id') as ggi, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'get_group_members') as ggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'delete_group_member') as dgm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'get_group_group_members') as gggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'delete_group_group_member') as dggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'rename_group') as rg:
            ggm.return_value = [{'email': 'body@sftests.com'},
                                {'email': 'body2@sftests.com'},
                                {'email': self.conf.admin['email']}]
            gggm.return_value = []
            logs = o.delete(**kwargs)
            self.assertEqual(len(ggi.call_args_list), 1)
            self.assertEqual(ggi.call_args_list[0],
                             call('space/g1'))
            self.assertEqual(len(ggm.call_args_list), 1)
            self.assertEqual(len(dgm.call_args_list), 3)
            self.assertEqual(len(rg.call_args_list), 1)
            self.assertListEqual([call('space/g1', 'body@sftests.com'),
                                  call('space/g1', 'body2@sftests.com'),
                                  call('space/g1', self.conf.admin['email'])],
                                 dgm.call_args_list)
            self.assertEqual(len(logs), 0)

        with patch('managesf.services.gerrit.utils.'
                   'GerritClient.get_group_id') as ggi, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'get_group_members') as ggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'delete_group_member') as dgm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'get_group_group_members') as gggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'delete_group_group_member') as dggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'rename_group') as rg:

            ggm.return_value = []
            ggi.return_value = '666'
            gggm.return_value = [{'name': 'included_group'}]
            logs = o.delete(**kwargs)
            self.assertEqual(len(ggi.call_args_list), 1)
            self.assertEqual(len(gggm.call_args_list), 1)
            self.assertEqual(len(rg.call_args_list), 1)
            self.assertEqual(gggm.call_args_list[0],
                             call('666'))
            self.assertEqual(len(dggm.call_args_list), 1)
            self.assertEqual(call('666', 'included_group'),
                             dggm.call_args_list[0])
            self.assertEqual(len(logs), 0)

        with patch('managesf.services.gerrit.utils.'
                   'GerritClient.get_group_id') as ggi, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'get_group_members') as ggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'delete_group_member') as dgm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'get_group_group_members') as gggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'delete_group_group_member') as dggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'rename_group'):

            ggm.return_value = [{'email': 'body@sftests.com'},
                                {'email': 'body2@sftests.com'},
                                {'email': self.conf.admin['email']}]
            dgm.side_effect = Exception('Random Error')
            gggm.return_value = []
            logs = o.delete(**kwargs)
            self.assertIn('Group delete [del member: body@sftests.com]: '
                          'err API returned Random Error', logs)
            self.assertIn('Group delete [del member: body2@sftests.com]: '
                          'err API returned Random Error', logs)
            self.assertIn('Group delete [del member: %s]: '
                          'err API returned Random Error' % (
                              self.conf.admin['email']), logs)
            self.assertEqual(len(logs), 3)

    def test_update(self):
        o = GroupOps(self.conf, None)

        kwargs = {'name': 'space/g1',
                  'description': 'An awesome project',
                  'members': ['body@sftests.com', 'body2@sftests.com']}

        with patch('managesf.services.gerrit.utils.'
                   'GerritClient.get_group_id'), \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'get_group_members') as ggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'add_group_member') as agm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'delete_group_member') as dgm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'get_group_group_members') as gggm, \
                patch('managesf.services.gerrit.utils.GerritClient.'
                      'delete_group_group_member') as dggm, \
                patch.object(GroupOps, 'group_update_description') as gup:
            ggm.return_value = [{'email': 'body3@sftests.com'},
                                {'id': 'John Doe'}]
            gggm.return_value = [{'name': 'included_group'}]
            logs = o.update(**kwargs)
            self.assertEqual(len(agm.call_args_list), 2)
            self.assertIn(call('body2@sftests.com', 'space/g1'),
                          agm.call_args_list)
            self.assertIn(call('body@sftests.com', 'space/g1'),
                          agm.call_args_list)
            self.assertEqual(len(dgm.call_args_list), 1)
            self.assertEqual(call('space/g1', 'body3@sftests.com'),
                             dgm.call_args_list[0])
            self.assertEqual(len(gup.call_args_list), 1)
            self.assertEqual(call('space/g1', 'An awesome project'),
                             gup.call_args_list[0])
            self.assertTrue(gggm.called)
            self.assertTrue(dggm.called)
            self.assertEqual(len(logs), 0)

    def test_extra_validations(self):
        kwargs = {'name': 'space/g1',
                  'members': ['body@sftests.com', 'body2@sftests.com']}
        o = GroupOps(self.conf, None)
        with patch('managesf.services.gerrit.utils.'
                   'GerritClient.get_account') as ga:
            ga.return_value = {}
            logs = o.extra_validations(**kwargs)
            self.assertEqual(len(ga.call_args_list), 2)
            self.assertEqual(len(logs), 0)
        with patch('managesf.services.gerrit.utils.'
                   'GerritClient.get_account') as ga:
            ga.side_effect = utils.NotFound()
            logs = o.extra_validations(**kwargs)
            self.assertEqual(len(ga.call_args_list), 2)
            self.assertEqual(len(logs), 2)
            self.assertIn('Check group members [body@sftests.com does '
                          'not exists]: err API unable to find the member',
                          logs)
            self.assertIn('Check group members [body2@sftests.com '
                          'does not exists]: err API unable to '
                          'find the member', logs)

    def test_get_all(self):
        o = GroupOps(self.conf, None)

        def fake_get_groups():
            return {
                'g1': {
                    'description': 'd1',
                    'group_id': 1,
                    },
                'g2': {
                    'description': 'd2',
                    'group_id': 2,
                    },
                'Administrators': {
                    'description': 'Gerrit Admin groups',
                    'group_id': 3,
                    },
                'Service Users': {
                    'description': 'Gerrit Service Users groups',
                    'group_id': 4,
                    },
            }

        def fake_get_group_members(group_id):
            groups = {
                '1': [
                    {'email': 'user1@sftests.com'},
                    {'email': 'user3@sftests.com'},
                    {'id': 'John Doe'}
                ],
                '2': [
                    {'email': 'user2@sftests.com'}
                ],
                '3': [
                    {'email': 'user2@sftests.com'}
                ],
                '4': [
                    {'email': 'user2@sftests.com'}
                ],
            }
            return groups[group_id]

        with patch('managesf.services.gerrit.utils.'
                   'GerritClient.get_groups') as gg, \
                patch('managesf.services.gerrit.utils.'
                      'GerritClient.get_group_members') as ggm:
            gg.side_effect = fake_get_groups
            ggm.side_effect = fake_get_group_members
            logs, g_tree = o.get_all()
            self.assertListEqual(logs, [])
            self.assertIn('g1', g_tree['groups'].keys())
            self.assertIn('g2', g_tree['groups'].keys())
            self.assertEqual(len(g_tree['groups'].keys()), 2)
            self.assertIn('user1@sftests.com',
                          g_tree['groups']['g1']['members'])
            self.assertIn('user3@sftests.com',
                          g_tree['groups']['g1']['members'])
            self.assertEqual(len(g_tree['groups']['g1']['members']), 2)
            self.assertIn('user2@sftests.com',
                          g_tree['groups']['g2']['members'])
            self.assertEqual(len(g_tree['groups']['g2']['members']), 1)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import git
import yaml
try:
    from yaml import CSafeDumper as YDumper
except ImportError:
    from yaml import SafeDumper as YDumper
import tempfile


def prepare_git_repo(db_path):
    repo_path = tempfile.mkdtemp()
    db_path.append(repo_path)
    repo = git.Git(repo_path)
    repo.init()
    return repo_path


def prepare_db_env(db_path):
    clone_path = tempfile.mkdtemp()
    fd, cache_path = tempfile.mkstemp()
    os.close(fd)
    db_path.append(clone_path)
    db_path.append(cache_path)
    return clone_path, cache_path


def add_yaml_data(repo_path, data, free_style=False):
    repo = git.Git(repo_path)
    sub_dir = "resources"
    db_path = os.path.join(repo_path, sub_dir)
    if not os.path.isdir(db_path):
        os.mkdir(db_path)
    filename = "%s.yaml" % id(data)
    if not free_style:
        with open(os.path.join(db_path, filename), 'w') as dbfile:
            yaml.dump(data,
                      dbfile,
                      allow_unicode=True,
                      default_flow_style=False,
                      Dumper=YDumper)
    else:
        open(os.path.join(db_path, filename), 'w').write(data)
    repo.execute(['git', 'add', sub_dir])
    repo.update_environment(GIT_AUTHOR_EMAIL='test@test.com')
    repo.update_environment(GIT_AUTHOR_NAME='test')
    repo.update_environment(GIT_COMMITTER_EMAIL='test@test.com')
    repo.update_environment(GIT_COMMITTER_NAME='test')
    repo.execute(['git', 'commit', '-m', 'add %s' % filename])
    return repo_path



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import yaml
import os
from unittest import TestCase
from webtest import TestApp
import tempfile

from pecan import load_app

from managesf.tests import dummy_conf
from managesf import policy
from managesf.policies import base


""""Test the policy engine"""


class TestPolicyEngine(TestCase):
    def setUp(self):
        c = dummy_conf()
        self.config = {'services': c.services,
                       'gerrit': c.gerrit,
                       'app': c.app,
                       'admin': c.admin,
                       'sqlalchemy': c.sqlalchemy,
                       'managesf': c.managesf,
                       'policy': c.policy,
                       'api': c.api, }
        self.app = TestApp(load_app(self.config))

    def tearDown(self):
        # Remove the sqlite db
        os.unlink(self.config['sqlalchemy']['url'][len('sqlite:///'):])

    def test_config_policies(self):
        """Test the default config endpoint policies"""
        credentials = {}
        target = {}
        self.assertFalse(policy.authorize('managesf.config:get',
                                          target, credentials))
        credentials = {'username': 'BirdPerson'}
        self.assertTrue(policy.authorize('managesf.config:get',
                                         target, credentials))

    def test_hooks_policies(self):
        """Test the default hooks endpoint policies"""
        credentials = {}
        target = {}
        self.assertFalse(policy.authorize('managesf.hooks:trigger',
                                          target, credentials))
        credentials = {'username': 'RickSanchez'}
        self.assertFalse(policy.authorize('managesf.hooks:trigger',
                                          target, credentials))
        credentials = {'username': 'admin'}
        self.assertTrue(policy.authorize('managesf.hooks:trigger',
                                         target, credentials))
        credentials = {'username': 'SF_SERVICE_USER'}
        self.assertTrue(policy.authorize('managesf.hooks:trigger',
                                         target, credentials))

    def test_localuser_policies(self):
        """Test the default localuser endpoint policies"""
        credentials = {}
        target = {}
        self.assertFalse(policy.authorize('managesf.localuser:get',
                                          target, credentials))
        self.assertFalse(policy.authorize('managesf.localuser:create_update',
                                          target, credentials))
        self.assertFalse(policy.authorize('managesf.localuser:delete',
                                          target, credentials))
        self.assertTrue(policy.authorize('managesf.localuser:bind',
                                         target, credentials))
        credentials = {'username': 'RickSanchez',
                       'groups': []}
        self.assertTrue(policy.authorize('managesf.localuser:get',
                                         target, credentials))
        self.assertFalse(policy.authorize('managesf.localuser:create_update',
                                          target, credentials))
        self.assertFalse(policy.authorize('managesf.localuser:delete',
                                          target, credentials))
        self.assertTrue(policy.authorize('managesf.localuser:bind',
                                         target, credentials))
        target = {'username': 'RickSanchez'}
        self.assertTrue(policy.authorize('managesf.localuser:create_update',
                                         target, credentials))
        self.assertTrue(policy.authorize('managesf.localuser:delete',
                                         target, credentials))
        target = {'username': 'Morty'}
        self.assertTrue(policy.authorize('managesf.localuser:get',
                                         target, credentials))
        self.assertFalse(policy.authorize('managesf.localuser:create_update',
                                          target, credentials))
        self.assertFalse(policy.authorize('managesf.localuser:delete',
                                          target, credentials))
        credentials['username'] = 'Morty'
        self.assertTrue(policy.authorize('managesf.localuser:create_update',
                                         target, credentials))
        self.assertTrue(policy.authorize('managesf.localuser:delete',
                                         target, credentials))

    def test_resources_policies(self):
        """Test the default resources endpoint policies"""
        credentials = {}
        self.assertTrue(policy.authorize('managesf.resources:get',
                                         {}, credentials))
        self.assertFalse(policy.authorize('managesf.resources:validate',
                                          {}, credentials))
        self.assertFalse(policy.authorize('managesf.resources:apply',
                                          {}, credentials))
        credentials = {'username': 'shimajiro'}
        self.assertTrue(policy.authorize('managesf.resources:get',
                                         {}, credentials))
        self.assertFalse(policy.authorize('managesf.resources:validate',
                                          {}, credentials))
        self.assertFalse(policy.authorize('managesf.resources:apply',
                                          {}, credentials))
        credentials = {'username': 'admin'}
        self.assertTrue(policy.authorize('managesf.resources:get',
                                         {}, credentials))
        self.assertTrue(policy.authorize('managesf.resources:validate',
                                         {}, credentials))
        self.assertTrue(policy.authorize('managesf.resources:apply',
                                         {}, credentials))
        credentials = {'username': 'SF_SERVICE_USER'}
        self.assertTrue(policy.authorize('managesf.resources:get',
                                         {}, credentials))
        self.assertTrue(policy.authorize('managesf.resources:validate',
                                         {}, credentials))
        self.assertTrue(policy.authorize('managesf.resources:apply',
                                         {}, credentials))

    def test_default_policies(self):
        """Test the default policies that come with a default deployment"""
        credentials = {}
        target = {}
        try:
            admin_account = self.config.admin['name']
        except AttributeError:
            admin_account = 'admin'
        self.assertTrue(policy.authorize('any',
                                         target, credentials))
        self.assertFalse(policy.authorize('none',
                                          target, credentials))
        self.assertFalse(policy.authorize('authenticated_api',
                                          target, credentials))
        credentials = {'username': 'Morty'}
        self.assertTrue(policy.authorize('any',
                                         target, credentials))
        self.assertFalse(policy.authorize('none',
                                          target, credentials))
        self.assertTrue(policy.authorize('authenticated_api',
                                         target, credentials))
        self.assertFalse(policy.authorize('admin_api',
                                          target, credentials))
        credentials = {'username': admin_account}
        self.assertTrue(policy.authorize('authenticated_api',
                                         target, credentials))
        self.assertTrue(policy.authorize('any',
                                         target, credentials))
        self.assertFalse(policy.authorize('none',
                                          target, credentials))
        self.assertTrue(policy.authorize('admin_api',
                                         target, credentials))
        self.assertTrue(policy.authorize('admin_or_service',
                                         target, credentials))
        self.assertFalse(policy.authorize('owner_api',
                                          target, credentials))
        self.assertTrue(policy.authorize('admin_or_owner',
                                         target, credentials))
        self.assertFalse(policy.authorize('contributor_api',
                                          target, credentials))
        credentials = {'username': base.SERVICE_USER}
        self.assertTrue(policy.authorize('authenticated_api',
                                         target, credentials))
        self.assertTrue(policy.authorize('any',
                                         target, credentials))
        self.assertFalse(policy.authorize('none',
                                          target, credentials))
        self.assertFalse(policy.authorize('admin_api',
                                          target, credentials))
        self.assertTrue(policy.authorize('admin_or_service',
                                         target, credentials))
        self.assertFalse(policy.authorize('owner_api',
                                          target, credentials))
        credentials = {'username': 'Jerry'}
        target = {'username': 'Summer'}
        self.assertTrue(policy.authorize('authenticated_api',
                                         target, credentials))
        self.assertTrue(policy.authorize('any',
                                         target, credentials))
        self.assertFalse(policy.authorize('none',
                                          target, credentials))
        self.assertFalse(policy.authorize('admin_api',
                                          target, credentials))
        self.assertFalse(policy.authorize('owner_api',
                                          target, credentials))
        target = {'username': 'Jerry'}
        self.assertTrue(policy.authorize('owner_api',
                                         target, credentials))
        credentials['groups'] = ['p0-dev', ]
        target = {'project': 'p1'}
        self.assertTrue(policy.authorize('any',
                                         target, credentials))
        self.assertFalse(policy.authorize('dev_api',
                                          target, credentials))
        self.assertFalse(policy.authorize('core_api',
                                          target, credentials))
        self.assertFalse(policy.authorize('ptl_api',
                                          target, credentials))
        self.assertFalse(policy.authorize('contributor_api',
                                          target, credentials))
        target = {'project': 'p0'}
        self.assertTrue(policy.authorize('dev_api',
                                         target, credentials))
        self.assertFalse(policy.authorize('core_api',
                                          target, credentials))
        self.assertFalse(policy.authorize('ptl_api',
                                          target, credentials))
        self.assertTrue(policy.authorize('contributor_api',
                                         target, credentials))
        credentials['groups'] = ['p0-core', ]
        self.assertFalse(policy.authorize('dev_api',
                                          target, credentials))
        self.assertTrue(policy.authorize('core_api',
                                         target, credentials))
        self.assertFalse(policy.authorize('ptl_api',
                                          target, credentials))
        self.assertTrue(policy.authorize('contributor_api',
                                         target, credentials))
        credentials['groups'] = ['p0-ptl', ]
        self.assertFalse(policy.authorize('dev_api',
                                          target, credentials))
        self.assertFalse(policy.authorize('core_api',
                                          target, credentials))
        self.assertTrue(policy.authorize('ptl_api',
                                         target, credentials))
        self.assertTrue(policy.authorize('contributor_api',
                                         target, credentials))


class TestPolicyEngineFromFile(TestCase):
    def setUp(self):
        c = dummy_conf()
        self.config = {'services': c.services,
                       'gerrit': c.gerrit,
                       'app': c.app,
                       'admin': c.admin,
                       'sqlalchemy': c.sqlalchemy,
                       'managesf': c.managesf,
                       'policy': c.policy, }
        pol_file = tempfile.mkstemp()[1] + '.yaml'
        with open(pol_file, 'w') as p:
            yaml.dump(
                {"is_morty": "username:morty",
                 "morty_api": "rule:is_morty"},
                p, default_flow_style=False)
        self.config['policy']['policy_file'] = pol_file
        self.app = TestApp(load_app(self.config))

    def test_file_policies(self):
        """Test that the specified policies are taken into account"""
        credentials = {}
        target = {}
        try:
            admin_account = self.config.admin['name']
        except AttributeError:
            admin_account = 'admin'
        # make sure default rules are there
        self.assertFalse(policy.authorize('admin_api',
                                          target, credentials))
        self.assertTrue(policy.authorize('admin_api',
                                         target,
                                         {'username': admin_account}))
        self.assertFalse(policy.authorize('morty_api',
                                          target, credentials))
        credentials['username'] = 'Rick'
        self.assertFalse(policy.authorize('admin_api',
                                          target, credentials))
        self.assertFalse(policy.authorize('morty_api',
                                          target, credentials))
        credentials['username'] = 'morty'
        self.assertFalse(policy.authorize('admin_api',
                                          target, credentials))
        self.assertTrue(policy.authorize('morty_api',
                                         target, credentials))

    def test_change_in_file_policies(self):
        pol_file = self.config['policy']['policy_file']
        with open(pol_file, 'w') as p:
            yaml.dump(
                {"is_rick": "username:Rick",
                 "rick_api": "rule:is_rick"},
                p, default_flow_style=False)
        credentials = {}
        target = {}
        try:
            admin_account = self.config.admin['name']
        except AttributeError:
            admin_account = 'admin'
        # make sure default rules are there
        self.assertFalse(policy.authorize('admin_api',
                                          target, credentials))
        self.assertFalse(policy.authorize('rick_api',
                                          target, credentials))
        credentials['username'] = 'Rick'
        self.assertFalse(policy.authorize('admin_api',
                                          target, credentials))
        self.assertTrue(policy.authorize('rick_api',
                                         target, credentials))
        credentials['username'] = 'morty'
        self.assertFalse(policy.authorize('admin_api',
                                          target, credentials))
        self.assertFalse(policy.authorize('rick_api',
                                          target, credentials))
        credentials['username'] = admin_account
        self.assertTrue(policy.authorize('admin_api',
                                         target, credentials))
        self.assertFalse(policy.authorize('rick_api',
                                          target, credentials))
        # set back to normal
        with open(pol_file, 'w') as p:
            yaml.dump(
                {"is_morty": "username:morty",
                 "morty_api": "rule:is_morty"},
                p, default_flow_style=False)

    def tearDown(self):
        # Remove the sqlite db
        os.unlink(self.config['sqlalchemy']['url'][len('sqlite:///'):])
        os.unlink(self.config['policy']['policy_file'])



#!/usr/bin/env python
#
# Copyright (C) 2017 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""Sample resources tree from a test instance."""


SAMPLE_RESOURCES_TREE = {
 'hash': '1234ab',
 'resources': {'acls': {'config-acl': {'file': '[...]',
                                       'groups': ['config-ptl',
                                                  'config-core']},
                        'dummy_project1-acl': {'file': '[...]',
                                               'groups': ['dummy_project1-core',  # noqa
                                                          'dummy_project1-ptl']},  # noqa
                        'dummy_project2-acl': {'file': '[...]',
                                               'groups': ['dummy_project2-core',  # noqa
                                                          'dummy_project2-ptl']},  # noqa
                        'dummy_project3-acl': {'file': '[...]',
                                               'groups': ['dummy_project3-core',  # noqa
                                                          'dummy_project3-ptl']},  # noqa
                        'dummy_project4-acl': {'file': '[...]',
                                               'groups': ['dummy_project4-core',  # noqa
                                                          'dummy_project4-ptl']},  # noqa
                        'dummy_project6-acl': {'file': '[...]',
                                               'groups': ['dummy_project6-core',  # noqa
                                                          'dummy_project6-ptl']},  # noqa
                        'dummy_project86-acl': {'file': '[...]',
                                                'groups': ['dummy_project86-core',  # noqa
                                                           'dummy_project86-ptl']},  # noqa
                        'tdpw-acl': {'file': '[...]',
                                     'groups': ['tdpw-core', 'tdpw-ptl']}},
               'groups': {'config-core': {'description': 'Team core for the config repo',  # noqa
                                          'members': []},
                          'config-ptl': {'description': 'Team lead for the config repo',  # noqa
                                         'members': ['admin@sftests.com']},
                          'dummy_project1-core': {'description': 'Core developers for project dummy_project1',  # noqa
                                                  'members': ['admin@sftests.com']},  # noqa
                          'dummy_project1-ptl': {'description': 'Project team lead for project dummy_project1',  # noqa
                                                 'members': ['admin@sftests.com']},  # noqa
                          'dummy_project2-core': {'description': 'Core developers for project dummy_project2',  # noqa
                                                  'members': ['admin@sftests.com']},  # noqa
                          'dummy_project2-ptl': {'description': 'Project team lead for project dummy_project2',  # noqa
                                                 'members': ['admin@sftests.com']},  # noqa
                          'dummy_project3-core': {'description': 'Core developers for project dummy_project3',  # noqa
                                                  'members': ['admin@sftests.com']},  # noqa
                          'dummy_project3-ptl': {'description': 'Project team lead for project dummy_project3',  # noqa
                                                 'members': ['admin@sftests.com']},  # noqa
                          'dummy_project4-core': {'description': 'Core developers for project dummy_project4',  # noqa
                                                  'members': ['admin@sftests.com']},  # noqa
                          'dummy_project4-ptl': {'description': 'Project team lead for project dummy_project4',  # noqa
                                                 'members': ['admin@sftests.com']},  # noqa
                          'dummy_project6-core': {'description': 'Core developers for project dummy_project6',  # noqa
                                                  'members': ['admin@sftests.com']},  # noqa
                          'dummy_project6-ptl': {'description': 'Project team lead for project dummy_project6',  # noqa
                                                 'members': ['admin@sftests.com']},  # noqa
                          'dummy_project86-core': {'description': 'Core developers for project dummy_project86',  # noqa
                                                   'members': ['admin@sftests.com']},  # noqa
                          'dummy_project86-ptl': {'description': 'Project team lead for project dummy_project86',  # noqa
                                                  'members': ['admin@sftests.com']},  # noqa
                          'tdpw-core': {'description': 'Core developers for project tdpw',  # noqa
                                        'members': ['adev@softwarefactory-project.io',  # noqa
                                                    'alead@softwarefactory-project.io'],  # noqa
                                        'name': 'tdpw-core'},
                          'tdpw-ptl': {'description': 'Project team lead for project tdpw',  # noqa
                                       'members': ['alead@softwarefactory-project.io'],  # noqa
                                       'name': 'tdpw-ptl'}},
               'projects': {'dummy_project1': {'description': 'Project dummy_project1',  # noqa
                                               'issue-tracker': 'SFStoryboard',
                                               'source-repositories': ['dummy_project1']},  # noqa
                            'dummy_project2': {'description': 'Project dummy_project2',  # noqa
                                               'issue-tracker': 'SFStoryboard',
                                               'source-repositories': ['dummy_project2']},  # noqa
                            'dummy_project3': {'description': 'Project dummy_project3',  # noqa
                                               'issue-tracker': 'SFStoryboard',
                                               'source-repositories': ['dummy_project3']},  # noqa
                            'dummy_project4': {'description': 'Project dummy_project4',  # noqa
                                               'issue-tracker': 'SFStoryboard',
                                               'source-repositories': ['dummy_project4']},  # noqa
                            'dummy_project6': {'description': 'Project dummy_project6',  # noqa
                                               'issue-tracker': 'SFStoryboard',
                                               'source-repositories': ['dummy_project6']},  # noqa
                            'dummy_project86': {'description': 'Project dummy_project86',  # noqa
                                                'issue-tracker': 'SFStoryboard',  # noqa
                                                'source-repositories': ['dummy_project86']},  # noqa
                            'internal': {'description': 'Internal configuration project',  # noqa
                                         'issue-tracker': 'SFStoryboard',
                                         'source-repositories': ['config']},
                            'my_project': {'contacts': ['boss@project.com'],
                                           'description': 'plop',
                                           'documentation': 'http://doc.project.com',  # noqa
                                           'issue-tracker': 'SFStoryboard',
                                           'mailing-lists': ['ml@project.com'],
                                           'source-repositories': ['sexbobomb'],  # noqa
                                           'website': 'http://project.com'},
                            'tdpw-project': {'description': 'The is a demo RPM distribution project to experiment',  # noqa
                                             'issue-tracker': 'SFStoryboard',
                                             'name': 'tdpw',
                                             'review-dashboard': 'default',
                                             'source-repositories': ['tdpw/python-readerlib',  # noqa
                                                                     'tdpw/python-readerlib-distgit',  # noqa
                                                                     'tdpw/reader',  # noqa
                                                                     'tdpw/reader-distgit',  # noqa
                                                                     'tdpw/reader-ansible',  # noqa
                                                                     'tdpw/reader-ansible-distgit',  # noqa
                                                                     'tdpw/tdpw-installer',  # noqa
                                                                     'tdpw/tdpw-installer-distgit',  # noqa
                                                                     'tdpw/tdpw-info']}},  # noqa
               'repos': {'config': {'acl': 'config-acl',
                                    'description': 'Config repository (Do not delete it)'},  # noqa
                         'dummy_project1': {'acl': 'dummy_project1-acl',
                                            'description': 'Code repository for dummy_project1'},  # noqa
                         'dummy_project2': {'acl': 'dummy_project2-acl',
                                            'description': 'Code repository for dummy_project2'},  # noqa
                         'dummy_project3': {'acl': 'dummy_project3-acl',
                                            'description': 'Code repository for dummy_project3'},  # noqa
                         'dummy_project4': {'acl': 'dummy_project4-acl',
                                            'description': 'Code repository for dummy_project4'},  # noqa
                         'dummy_project6': {'acl': 'dummy_project6-acl',
                                            'description': 'Code repository for dummy_project6'},  # noqa
                         'dummy_project86': {'acl': 'dummy_project86-acl',
                                             'description': 'Code repository for dummy_project86'},  # noqa
                         'sexbobomb': {'acl': 'config-acl',
                                       'description': 'oh yeah'},
                         'tdpw/python-readerlib': {'acl': 'tdpw-acl',
                                                   'description': 'Python library of the Reader project',  # noqa
                                                   'name': 'tdpw/python-readerlib'},  # noqa
                         'tdpw/python-readerlib-distgit': {'acl': 'tdpw-acl',
                                                           'description': 'RPM packaging for python-readerlib',  # noqa
                                                           'name': 'tdpw/python-readerlib-distgit'},  # noqa
                         'tdpw/reader': {'acl': 'tdpw-acl',
                                         'description': 'The Reader server',
                                         'name': 'tdpw/reader'},
                         'tdpw/reader-ansible': {'acl': 'tdpw-acl',
                                                 'description': 'The Ansible role of the Reader server',  # noqa
                                                 'name': 'tdpw/reader-ansible'},  # noqa
                         'tdpw/reader-ansible-distgit': {'acl': 'tdpw-acl',
                                                         'description': 'RPM packaging for the Reader Ansible role',  # noqa
                                                         'name': 'tdpw/reader-ansible-distgit'},  # noqa
                         'tdpw/reader-distgit': {'acl': 'tdpw-acl',
                                                 'description': 'RPM packaging for the Reader server',  # noqa
                                                 'name': 'tdpw/reader-distgit'},  # noqa
                         'tdpw/tdpw-info': {'acl': 'tdpw-acl',
                                            'description': 'tdpw Distribution info repository',  # noqa
                                            'name': 'tdpw/tdpw-info'},
                         'tdpw/tdpw-installer': {'acl': 'tdpw-acl',
                                                 'description': 'The installer for the tdpw RPM Distribution',  # noqa
                                                 'name': 'tdpw/tdpw-installer'},  # noqa
                         'tdpw/tdpw-installer-distgit': {'acl': 'tdpw-acl',
                                                         'description': 'RPM packaging for the tdpw Distribution installer',  # noqa
                                                         'name': 'tdpw/tdpw-installer-distgit'}}}  # noqa
}



#!/usr/bin/env python

from unittest.mock import patch, MagicMock

m_mock = MagicMock()

# Patch all useless module imports for generating the doc
modules = {'managesf.services.gerrit': m_mock,
           'git': m_mock,
           'yaml': m_mock,
           'deepdiff': m_mock,
           'requests': m_mock,
           'requests.exceptions': m_mock,
           'git.config': m_mock,
           'pecan': m_mock,
           'pynotedb': m_mock,
           'sqlalchemy': m_mock,
           'sqlalchemy.orm': m_mock,
           'sqlalchemy.ext': m_mock,
           'sqlalchemy.orm.exc': m_mock,
           'sqlalchemy.ext.declarative': m_mock,
           'storyboardclient': m_mock,
           'storyboardclient._apiclient': m_mock,
           'storyboardclient._apiclient.exceptions': m_mock,
           'storyboardclient.v1': m_mock,
           'storyboardclient.v1.client': m_mock}

m_patcher = patch.dict('sys.modules', modules)
m_patcher.start()

from managesf.model.yamlbkd.resources.gitrepository import GitRepository
from managesf.model.yamlbkd.resources.project import Project
from managesf.model.yamlbkd.resources.gitacls import ACL
from managesf.model.yamlbkd.resources.group import Group

from managesf.model.yamlbkd.engine import MAPPING


def render_resource(cls):
    print()
    print(cls.MODEL_TYPE)
    print("^"*len(cls.MODEL_TYPE))
    print()
    print(cls.DESCRIPTION)
    print()
    print("Below are the list of keys available for this resource.")
    print()
    for key, details in cls.MODEL.items():
        if key == "name":
            key = "name (Overwritten by the resource ID)"
            print()
            print(key)
            print('"'*len(key))
            print("* **Description:** %s" % details[5])
            print("* **Authorized value:** RE(%s)" % details[1])
            print("* **Mandatory key:** %s" % details[2])
        else:
            print()
            print(key)
            print('"'*len(key))
            print("* **Description:** %s" % details[5])
            print("* **Type:** %s" % str(details[0]))
            print("* **Authorized value:** RE(%s)" % str(details[1]))
            print("* **Mandatory key:** %s" % details[2])
            print("* **Mutable key:** %s" % details[4])
            if not details[2]:
                default = details[3]
                if isinstance(default, str) and not len(default):
                    default = "\"\""
                print("* **Default value:** %s" % default)

if __name__ == '__main__':
    print(".. _config-resources-model:")
    print()
    print("Available resources models")
    print("==========================")
    print()
    print("Containers / Resources mapping")
    print("------------------------------")
    for container, cls in MAPPING.items():
        print("* Model type: **%s** can only be "
              "defined in container: **%s**" % (
                  cls.MODEL_TYPE, container))
    print()
    print("Resources")
    print("---------")
    print()
    print("Note that the key **name** for every resource type"
          " is not mandatory. A resource is referenced via"
          " its resource's ID that will automatically become the resource"
          " **name** value. The resource's ID must validate the **name**"
          " value constraint of the related resource type. If you still"
          " want to specify the name then please make sure to have"
          " the resource ID equals to the resource name.")
    print()
    for cls in (Project, ACL, GitRepository, Group):
        render_resource(cls)



#!/usr/bin/env python
#
# Copyright (C) 2014 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import TestCase
from mock import patch

from managesf import sfauth


class Fake:
    cookies = {'auth_pubtkt': '1234'}


class FakeJSONResponse(object):
    def __init__(self, dic):
        self.dic = dic
        self.status_code = 200

    def json(self):
        return self.dic


class TestSFAuth(TestCase):
    def test_get_managesf_info(self):
        with patch('requests.get') as g:
            g.return_value = FakeJSONResponse({'service': {
                'name': 'managesf',
                'version': 'x.y.z',
                'services': ['gerrit', ]}})
            i = sfauth.get_managesf_info('https://auth.tests.dom')
            g.assert_called_with('https://auth.tests.dom/about/',
                                 allow_redirects=False,
                                 verify=True)
            self.assertEqual('managesf',
                             i['service']['name'])



#!/usr/bin/env python
#
# Copyright (C) 2015 Red Hat <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import abc
import logging
import six

from pecan import conf as pconf

from managesf.services import exceptions as exc


logger = logging.getLogger(__name__)


@six.add_metaclass(abc.ABCMeta)
class BaseHooksManager(object):
    """Abstract class handling hooks triggered by specific events in the CI
    workflow. These are mainly based on gerrit's own hook events."""
    def __init__(self, plugin):
        self.plugin = plugin

    def patchset_created(self, *args, **kwargs):
        """Called whenever a new patch is submitted. Expected arguments
        based on the gerrit hook with the same name:
        'change', 'is_draft', 'change_url', 'project',
        'branch', 'topic', 'uploader', 'commit', 'patchset',
        'commit_message'
        (The last one will be computed by the local hook script)"""
        raise exc.UnavailableActionError()

    def change_merged(self, *args, **kwargs):
        """Called whenever a new patch is merged into a project's master
        branch. Expected arguments based on the gerrit hook with the same name:
        'change', 'change_url', 'project',
        'branch', 'topic', 'submitter', 'commit', 'commit_message'
        (The last one will be computed by the local hook script)"""
        raise exc.UnavailableActionError()

    def __getattr__(self, hook):
        """Generic behavior for undefined hooks"""
        def _generic_hook(*args, **kwargs):
            msg = "[%s] undefined hook %s" % (self.plugin.service_name,
                                              hook)
            raise exc.UnavailableActionError(msg)
        return _generic_hook


@six.add_metaclass(abc.ABCMeta)
class BaseCRUDManager(object):
    def __init__(self, plugin):
        self.plugin = plugin

    def create(self, *args, **kwargs):
        """Creation operation"""
        raise exc.UnavailableActionError()

    def get(self, *args, **kwargs):
        """Fetching operation"""
        raise exc.UnavailableActionError()

    def update(self, *args, **kwargs):
        """Update operation"""
        raise exc.UnavailableActionError()

    def delete(self, *args, **kwargs):
        """Deletion operation"""
        raise exc.UnavailableActionError()


@six.add_metaclass(abc.ABCMeta)
class ProjectManager(BaseCRUDManager):
    """Abstract class handling CRUD operations on projects in Software
    Factory, if the service has a notion of projects."""


@six.add_metaclass(abc.ABCMeta)
class UserManager(BaseCRUDManager):
    """Abstract class handling CRUD operations on users in Software
    Factory, if the service has a notion of users."""

    # list of fields that cannot be updated. For example, it is not allowed
    # to change the username in gerrit.
    _immutable_fields_ = []

    @classmethod
    def check_forbidden_fields(cls, **kwargs):
        return list(set(cls._immutable_fields_) & set(kwargs.keys()))

    def update(self, uid, *args, **kwargs):
        """Update operation"""
        raise exc.UnavailableActionError()


@six.add_metaclass(abc.ABCMeta)
class GroupManager(BaseCRUDManager):
    """Abstract class handling CRUD operations on groups in Software
    Factory, if the service has a notion of groups."""

    # list of fields that cannot be updated. For example, it is not allowed
    # to change the group name in gerrit.
    _immutable_fields_ = []

    @classmethod
    def check_forbidden_fields(cls, **kwargs):
        return list(set(cls._immutable_fields_) & set(kwargs.keys()))

    def update(self, uid, *args, **kwargs):
        """Update operation"""
        raise exc.UnavailableActionError()


@six.add_metaclass(abc.ABCMeta)
class CodeReviewManager(BaseCRUDManager):
    """Abstract class handling code reviews operations for a given
    gerrit-like service."""


@six.add_metaclass(abc.ABCMeta)
class RoleManager(BaseCRUDManager):
    """Abstract class handling roles if the service can handle them"""

    @staticmethod
    def is_admin(user):
        return user == pconf.admin['name']


@six.add_metaclass(abc.ABCMeta)
class BaseServicePlugin(object):
    """Base plugin for a service that can be managed by Software Factory.
    """

    _config_section = "base"
    service_name = "base service"

    def __init__(self, conf):
        self._full_conf = conf
        try:
            self.configure_plugin(conf)
        except AttributeError:
            raise Exception(repr(conf))
        # place holders
        self.project = ProjectManager(self)
        self.user = UserManager(self)
        self.role = RoleManager(self)
        self.hooks = BaseHooksManager(self)

    def configure_plugin(self, conf):
        try:
            self.conf = getattr(conf, self._config_section, None)
        except KeyError:
            msg = ("The %s service is not available" % self._config_section)
            raise exc.ServiceNotAvailableError(msg)
        if not self.conf:
            msg = ("The %s service is not available" % self._config_section)
            raise exc.ServiceNotAvailableError(msg)

    def get_client(self, cookie=None):
        """returns a service client to be used by the managers."""


@six.add_metaclass(abc.ABCMeta)
class BaseCodeReviewServicePlugin(BaseServicePlugin):
    """Base plugin for a service managing code review systems. This adds
    code review operations like submitting a file for review to a project."""

    def __init__(self, conf):
        super(BaseCodeReviewServicePlugin, self).__init__(conf)
        self.review = CodeReviewManager(self)



# Copyright (C) 2015 Red Hat <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import logging
import pynotedb
import threading

from managesf.services import base
from managesf.services.gerrit import utils


class SFGerritUserManager(base.UserManager):

    log = logging.getLogger("managesf.GerritUserManager")

    def __init__(self, plugin):
        super(SFGerritUserManager, self).__init__(plugin)
        self.repo = None
        self.repo_lock = threading.Lock()

    def _add_account_as_external(self, account_id, username):
        """Inject username as external_ids.
        """
        with self.repo_lock:
            if 'keycloak' in self.plugin._full_conf['services']:
                scheme = pynotedb.scheme_keycloak
            else:
                # This is the scheme used by REMOTE_USER
                scheme = pynotedb.scheme_gerrit
            if self.repo is None:
                self.repo = pynotedb.mk_clone("ssh://gerrit/All-Users")
            try:
                pynotedb.add_account_external_id(
                    self.repo, username, str(account_id),
                    scheme)
            except Exception as e:
                self.log.exception(
                    "Couldn't insert user %s external_ids %s: %s",
                    account_id, username, e)
                raise

    def create(self, username, email, full_name, ssh_keys, cauth_id):
        self.log.debug(u"Creating account %s", username)
        # Special case for the admin user
        if cauth_id == 1:
            self.log.warning("Attempt to create the admin user. Skip.")
            return 1
        ssh_keys_valid = []
        # Need to clean ssh_keys as they start with incorrect data by default:
        #   e.g.: [{u'key': u'None'}]  or [u'key']
        if ssh_keys:
            for key in ssh_keys:
                if isinstance(key, str) and key != u"None":
                    ssh_keys_valid.append(key)
                elif key.get("key") and key.get('key') != u"None":
                    ssh_keys_valid.append(key.get("key"))
        client = self.plugin.get_client()
        data = {"name": str(full_name), "email": email}
        if ssh_keys_valid:
            data["ssh_key"] = ssh_keys_valid[0]
        try:
            user = client.create_account(username, data)
        except utils.GerritClientError as e:
            if e.status_code == 409:
                self.log.warning("Account %s already exists", username)
                # Convert create to update parameters...
                for o, n in (("name", "full_name"), ("ssh_key", "ssh_keys")):
                    data[n] = data.get(o)
                    try:
                        del data[o]
                    except Exception:
                        pass
                return self.update(uid=username, **data).get('_account_id')
            raise

        try:
            account_id = user.get('_account_id')
        except Exception:
            self.log.exception(u"Could not create user %s", user)
            raise

        self._add_account_as_external(account_id, username)
        # Add extra ssh keys
        if len(ssh_keys_valid) > 1:
            for key in ssh_keys_valid[1:]:
                client.add_pubkey(key, username)
        self.log.info(u'User %s created', username)
        return account_id

    def get(self, username):
        self.log.debug(u"Getting account %s", username)
        try:
            client = self.plugin.get_client()
            return client.get_account(username).get('_account_id')
        except utils.NotFound:
            return None

    def update(self, uid, username=None, full_name=None, email=None,
               ssh_keys=None, external_id=None, **infos):
        self.log.debug(u"Updating account %s", uid)
        client = self.plugin.get_client()
        user = client.get_account(uid, details=True)
        if email and isinstance(email, bytes):
            email = email.decode('utf-8')
        if full_name is not None and user.get("name") != full_name:
            client.update_account_name(uid, full_name)
        if email is not None and user.get("email") != email:
            if user.get("email"):
                client.delete_account_email(uid, user["email"])
            if email in user.get("secondary_emails", []):
                client.update_account_preferred_email(uid, email)
            else:
                client.add_account_email(uid, email)
        if ssh_keys is not None and ssh_keys != u'None':
            if not isinstance(ssh_keys, list):
                ssh_keys = [{"key": ssh_keys}]
            keys = []
            for key in ssh_keys:
                if key.get('key') and key.get('key') != u'None':
                    keys.append(key['key'])
            if keys:
                existing_keys = client.get_pubkeys(uid)
                for key in keys:
                    if [True for existing_key in existing_keys
                            if existing_key["encoded_key"].split()[0] in key]:
                        continue
                    client.add_pubkey(key, uid)
        return user

    def delete(self, username):
        self.log.warning("Can't delete the user %s, need manual intervention",
                         username)
        return



# Copyright (C) 2016 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import logging

from managesf.services import base
from managesf.services import exceptions as exc

logger = logging.getLogger(__name__)


class SFGerritGroupManager(base.GroupManager):

    def get_project_group_ids(self, client):
        project_groups = client.get_project_groups_id(client.get_projects())
        project_groups_ids = []
        for groups in project_groups.values():
            project_groups_ids.extend(groups['owners'])
            project_groups_ids.extend(groups['others'])
        return project_groups_ids

    def get(self, groupname=None, discard_pgroups=True):
        client = self.plugin.get_client()
        logger.info("[%s] Get group %s" % (self.plugin.service_name,
                                           groupname or "All groups"))
        ret = {}

        if groupname:
            gid = client.get_group_id(groupname)
            if not gid:
                raise exc.GroupNotFoundException("Unable to find group %s"
                                                 % groupname)
            if discard_pgroups:
                # We want to be sure the group is not a project group
                # referenced in refs.meta.config
                project_groups_ids = self.get_project_group_ids(client)
                if gid in project_groups_ids:
                    raise exc.GroupNotFoundException("Unable to find group %s"
                                                     % groupname)
            ret[groupname] = client.get_group_members(gid)
        else:
            project_groups_ids = self.get_project_group_ids(client)
            for groupname, details in client.get_groups().items():
                if details['id'] not in project_groups_ids:
                    ret[groupname] = {
                        'description': details['description'],
                        'members': client.get_group_members(details['id'])}
            for private in ("Administrators", "Service Users"):
                if private in ret:
                    del ret[private]

        return ret



# Copyright (C) 2017 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import TestCase

from managesf.api.v2 import base
from managesf.tests import dummy_conf


class TestAPIv2Service(TestCase):
    @classmethod
    def setupClass(cls):
        cls.conf = dummy_conf()

        class DummyService(base.BaseService):
            _config_section = "DummyService"
            service_name = "dummy service"

        cls.service = DummyService(cls.conf)

    @classmethod
    def tearDownClass(cls):
        pass

    def test_baseService(self):
        """Test loading configuration at service instantiation"""
        self.assertEqual(self.conf.DummyService['dummy_opt1'],
                         self.service.conf.get('dummy_opt1'))



#
# Copyright (C) 2016 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
# -*- coding: utf-8 -*-


"""the policy engine for managesf."""


import logging
import os.path

from pecan import conf
from oslo_policy import policy

from managesf import policies


logger = logging.getLogger(__name__)


_ENFORCER = None


class FakeOsloPolicy:
    def __init__(self, policy_file):
        self.policy_dirs = []
        self.policy_default_rule = None
        self.policy_file = policy_file


class FakeOslo:
    def __init__(self, policy_file):
        self.oslo_policy = FakeOsloPolicy(policy_file)

    def register_opts(self, *arg, **kwarg):
        return

    def find_file(self, *arg, **kwarg):
        return self.oslo_policy.policy_file


def reset():
    global _ENFORCER
    if _ENFORCER:
        _ENFORCER.clear()
        _ENFORCER = None


def init(policy_file=None, rules=None):
    """Init an Enforcer class.
       :param policy_file: Custom policy file to use, if none is specified,
                           `CONF.policy_file` will be used.
       :param rules: Default dictionary / Rules to use. It will be
                     considered just in the first instantiation.
    """

    global _ENFORCER
    if not _ENFORCER:
        _ENFORCER = policy.Enforcer(FakeOslo(policy_file),
                                    policy_file=policy_file,
                                    rules=rules,
                                    use_conf=False)
        _ENFORCER.register_defaults(policies.list_rules())
    if policy_file:
        _ENFORCER.load_rules(force_reload=True)
    register_rules(_ENFORCER)


def register_rules(enforcer):
    if not enforcer.rules:
        # Override defaults with the file rules
        for override in enforcer.file_rules.values():
            enforcer.registered_rules[override.name] = override.check
        for default in enforcer.registered_rules.values():
            if default.name not in enforcer.rules:
                enforcer.rules[default.name] = default.check


def authorize(rule_name, target, credentials):
    reset()
    try:
        policy_file = conf['policy'].get('policy_file')
    except KeyError:
        logger.info('Policy file not defined, going with default rules')
        policy_file = ''
    if not policy_file or not os.path.isfile(policy_file):
        msg = ('Policy file %s not found, initializing default policy '
               'engine (this is normal when bootstrapping '
               'Software Factory)')
        logger.info(msg % policy_file)
        init()
    else:
        init(policy_file=policy_file)
    try:
        result = _ENFORCER.enforce(rule_name, target, credentials,
                                   do_raise=False)
    except policy.PolicyNotRegistered:
        logger.error('Policy %s not registered' % rule_name)
        return -1
    except Exception:
        logger.debug('Policy check for %(rule)s failed with credentials '
                     '%(credentials)s' % {'rule': rule_name,
                                          'credentials': credentials})
        raise
    return result
